'use strict';

define([
	'app',
	'libraries/consts',
	], function(app, CONSTS) {
	app.controller("accounts",
		["$rootScope", "$scope", "$location", "$timeout", "$uibModal", "jetapi", "meta", "storage", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, $uibModal, jetapi, meta, storage, SweetAlert) {
				$rootScope.$emit('menuItem', 'Accounts');

				$scope.accounts = [];
				$scope.loadingAccounts = false;

				$scope.loaders = {
					quota: false,
					snapshot: false,
					clone: false,
					refresh: false
				};

				$scope.types = {};
				$scope.types[CONSTS.BACKUP_TYPE_FULL] = $scope.LANG.t("Full Account");
				$scope.types[CONSTS.BACKUP_TYPE_CONFIG] = $scope.LANG.t("cPanel Configuration");
				$scope.types[CONSTS.BACKUP_TYPE_FILES] = $scope.LANG.t("File");
				$scope.types[CONSTS.BACKUP_TYPE_DATABASES] = $scope.LANG.t("Database");
				$scope.types[CONSTS.BACKUP_TYPE_EMAILS] = $scope.LANG.t("Email Account");
				$scope.types[CONSTS.BACKUP_TYPE_CRON_JOBS] = $scope.LANG.t("Cron Job");
				$scope.types[CONSTS.BACKUP_TYPE_DNS_ZONES] = $scope.LANG.t("DNS Zone");
				$scope.types[CONSTS.BACKUP_TYPE_SSL_CERTIFICATES] = $scope.LANG.t("SSL Certificate");

				var metaAcct = meta.new("accounts");
				$scope.meta = metaAcct;
				$scope.meta.setPageSizes([25, 50, 100]);
				$scope.metaData = metaAcct.getData();

				metaAcct.setSortReverse(false);
				if(!metaAcct.getSortBy()) metaAcct.setSortBy("username");
				if(!metaAcct.getSortDirection()) metaAcct.setSortDirection("asc");
				metaAcct.setSortFields(["domain", "ip", "username", "disk_space_usage", "disk_files_usage", "backup_stats.127", "partition", "owner"]);
				metaAcct.setTotalItems($scope.accounts.length);

				jetapi.listQueuePriorities({}, function(response) {
					$scope.queuePriorities = {};
					var priorities = response.data.priorities;
					for(var i=0; i<priorities.length; i++) {
						if(priorities[i].default) $scope.defaultQueuePriory = priorities[i]._id;
						var id = priorities[i]._id;
						$scope.queuePriorities[id] = priorities[i].name;
					}
				});

				$scope.refreshStats = function() {

					if($scope.loaders.refresh) return;
					$scope.loaders.refresh = true;

					jetapi.updateAccountsBackupStats({}, function(response) {

						$scope.loaders.refresh = false;
						if(!response.success) {
							SweetAlert.error($scope.LANG.t("Error!"), response.message);
							return;
						}

						$scope.fetchData();
						SweetAlert.success($scope.LANG.t("Success!"), response.message);
					});
				};

				$scope.toggleBackupQuota = function (account) {

					$scope.loaders.quota = true;
					var newState = !account.backup_quota;
					var text = $scope.LANG.t("This account will continue backing up even if disk/inodes filter rule apply");
					if(!newState) text = $scope.LANG.t("This account will not backing up if disk/inodes filter rule apply");

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: text,
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							jetapi.manageAccount({ username: account.username, backupquota: newState }, function(response) {

								$scope.loaders.quota = false;

								if(response.success)
								{
									account.backup_quota = newState;
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
									return;
								}

								SweetAlert.error($scope.LANG.t("Failed!"), response.message);
							});
						}
						else
						{
							$scope.loaders.quota = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Enable backup quota cancelled"));
						}
					});
				};

				$scope.createSnapshots = function() {

					for(var i = 0; i < $scope.accounts.length; i++) {
						if($scope.accounts[i].checked) jetapi.addQueueSnapshot({ username: $scope.accounts[i].username });
					}

					SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("The selected account added to the snapshot queue successfully"));
				};

				$scope.createSnapshot = function(account) {

					$scope.loaders.snapshot = true;

					jetapi.addQueueSnapshot({ username: account.username }, function(response) {

						$scope.loaders.snapshot = false;

						if(response.success)
						{
							SweetAlert.success($scope.LANG.t("Success!"), response.message);
							return;
						}

						SweetAlert.error($scope.LANG.t("Failed!"), response.message);
					});
				};

				$scope.createClones = function () {

					var modalInstance = $uibModal.open({
						animation: true,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/cloneSelection.ptt',
						controller: 'cloneSelection',
						controllerAs: '$ctrl',
						resolve: {
							account: function() { return null; }
						}
					});

					modalInstance.result.then(function(cloneDetails) {

						if(!cloneDetails.destination) {
							SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("Please Choose destination to clone to"));
							return;
						}

						if(!cloneDetails.flag) {
							SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("Please Choose the account IP settings"));
							return;
						}

						for(var i = 0; i < $scope.accounts.length; i++) {
							if($scope.accounts[i].checked) jetapi.addQueueClone({
								username: $scope.accounts[i].username,
								destination_id: cloneDetails.destination,
								flag: cloneDetails.flag,
								backup_id: cloneDetails.backup
							});
						}

						SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("The selected account added to the snapshot queue successfully"));
					}, function () {});
				};

				$scope.createClone = function (account) {

					var modalInstance = $uibModal.open({
						animation: true,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/cloneSelection.ptt',
						controller: 'cloneSelection',
						controllerAs: '$ctrl',
						resolve: {
							account: function() { return account; }
						}
					});

					modalInstance.result.then(function(cloneDetails) {

						if(!cloneDetails.destination) {
							SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("Please Choose destination to clone to"));
							return;
						}

						if(!cloneDetails.flag) {
							SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("Please Choose the account IP settings"));
							return;
						}

						$scope.loaders.clone = true;

						var apiParams = {
							username: account.username,
							destination_id: cloneDetails.destination,
							flag: cloneDetails.flag,
							backup_id: cloneDetails.backup
						};

						jetapi.addQueueClone(apiParams, function(response) {

							$scope.loaders.clone = false;

							if(response.success)
							{
								SweetAlert.success($scope.LANG.t("Success!"), response.message);
								return;
							}
							SweetAlert.error($scope.LANG.t("Failed!"), response.message);
						});

					}, function () {});
				};

				$scope.viewBackups = function(account) {
					var backupMeta = storage.create("backup_manager_accounts_filter");
					backupMeta.set('account', account.username);
					backupMeta.save();
					$location.path('/backupManager/accounts');
				};

				$scope.viewPermissions = function(account) {
					var permissionStorage = storage.create("view_account_permissions");
					permissionStorage.set('account', account.username);
					permissionStorage.save();
					$location.path('/permissions/cpanel');

				};

				$scope.checkChanged = function () {
					for(var i = 0; i < $scope.accounts.length; i++)
					{
						if($scope.accounts[i].checked)
						{
							$scope.checked = true;
							return;
						}
					}

					$scope.checked = false;
				};
				$scope.setCheckAll = function() {
					for(var i = 0; i < $scope.accounts.length; i++) {
						$scope.accounts[i].checked = $scope.checkall;
					}
					$scope.checkChanged();
				};

				$scope.fetchSnapshotsSettings = function () {

					jetapi.getSettings({ section: '' })
				};
				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingAccounts = true;

					var apiParams = {
						sort: {},
						skip: metaAcct.getSkip(),
						limit: metaAcct.getPageSize(),
						find: { type: $scope.filter },
						filter: metaAcct.getFilter()
					};

					apiParams.sort[metaAcct.getSortBy()] = metaAcct.getSortDirectionInt();

					$scope.accounts = [];

					jetapi.listAccounts(apiParams, function(response) {
						if(!response.success) $scope.hasError = response.message;
						else $scope.hasError = false;
						metaAcct.setTotalItems(response.data.total);
						metaAcct.calculate(response.data.accounts);
						$scope.accounts = response.data.accounts;
						$scope.loadingAccounts = false;
					});
				};

				$timeout($scope.fetchData);
	}]);
});