'use strict';

define([
	'app'
	], function(app) {
	app.controller("alerts",
		["$rootScope", "$scope", "$location", "$timeout", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'Alerts');

				$scope.alerts = [];
				$scope.loadingAlerts = false;
				$scope.showPager = true;
				$scope.checkall = false;
				$scope.checked = false;
				$scope.filter = 'all';
				$scope.filterOptions = [
					{ label: $scope.LANG.t('All Alerts'), value: 'all' },
					{ label: $scope.LANG.t('UnAcknowledged Alerts'), value: 'acknowledge' },
					{ label: $scope.LANG.t('Information Alerts'), value: 'info' },
					{ label: $scope.LANG.t('Warning Alerts'), value: 'warning' },
					{ label: $scope.LANG.t('Critical Alerts'), value: 'critical' }
				];

				meta = meta.new("alerts");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("created");
				if(!meta.getSortDirection()) meta.setSortDirection("desc");
				meta.setSortFields(["created","level"]);
				meta.setTotalItems($scope.alerts.length);

				$scope.checkChanged = function () {
					for(var i = 0; i < $scope.alerts.length; i++)
					{
						if($scope.alerts[i].checked && !$scope.alerts[i].acknowledge)
						{
							$scope.checked = true;
							return;
						}
					}

					$scope.checked = false;
				};

				$scope.setCheckAll = function() {
					for(var i = 0; i < $scope.alerts.length; i++) {
						if(!$scope.alerts[i].acknowledge) $scope.alerts[i].checked = $scope.checkall;
					}
					$scope.checkChanged();
				};

				$scope.updateAlert = function(alert) {

					for(var i in $scope.alerts)
					{
						if($scope.alerts[i]._id === alert._id)
						{
							$scope.alerts[i] = alert;
							return false;
						}
					}
				};

				$scope.acknowledgeSelected = function() {
					var ids = [];

					for(var i in $scope.alerts)
					{
						if($scope.alerts[i].checked) ids.push($scope.alerts[i]._id);
					}

					if(!ids.length)
					{
						SweetAlert.error($scope.LANG.t("Error!"), $scope.LANG.t("No alert IDs selected"));
						return false;
					}

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("The selected alerts will be acknowledged!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							jetapi.acknowledgeAlerts({ _id: ids }, function(response) {

								if(response.success)
								{
									$scope.checkall = false;
									$scope.checked = false;
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
									return true;
								}

								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								return false;
							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Alerts acknowledge cancelled"));
						}

					});

				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingAlerts = true;

					var find = {};

					switch($scope.filter)
					{
						case 'acknowledge': find.acknowledge = 0; break;
						case 'info': find.level = 32; break;
						case 'warning': find.level = 64; break;
						case 'critical': find.level = 128; break;
					}

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: find,
						filter: meta.getFilter(),
						show_acknowledged: 1
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.alerts = [];

					jetapi.listAlerts(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.alerts);
						$scope.alerts = response.data.alerts;
						$scope.loadingAlerts = false;
					});
				};

				$timeout($scope.fetchData);
			}
		]
	);
});