'use strict';

define([
	'app',
	'libraries/consts'
	], function(app, CONSTS) {
	app.controller("backupJobs",
		["$rootScope", "$scope", "$location", "$timeout", "$interval", "$q", "jetapi", "meta", "filter", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, $interval, $q, jetapi, meta, filter, SweetAlert) {
				$rootScope.$emit('menuItem', 'BackupJobs');

				$scope.jobs = [];
				$scope.checkingJob = {};
				$scope.loadingJobs = false;
				$scope.filters = {};
				$scope.destinations = undefined;

				$scope.currentTime = parseInt(Date.now() / 1000);
				$interval(function () { $scope.currentTime = parseInt(Date.now() / 1000); }, 100);

				$scope.loaders = {
					state: false,
					quota: false,
					nextrun: false,
					duplicate: false,
					delete: false
				};

				$scope.status = undefined;

				meta = meta.new("backup_jobs");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("description");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["description","type","owner","retain","last_run","disabled"]);
				meta.setTotalItems($scope.jobs.length);

				$scope.checkProgress = function () {

					for(var i = 0; i < $scope.jobs.length; i++)
					{
						if(!$scope.jobs[i].running) continue;

						(function (i) {
							jetapi.getBackupJob({ _id: $scope.jobs[i]._id }, function(response) {
								// update all job data
								for(var a in response.data) {
									$scope.jobs[i][a] = response.data[a];
									$scope.jobs[i].next_run_int = $scope.jobs[i].next_run ? parseInt((new Date($scope.jobs[i].next_run)).getTime() / 1000) : 0;
								}
							});
						})(i);
					}
				};

				$scope.checkInterval = $interval($scope.checkProgress, 3000);

				$scope.$on("$destroy", function() {
					$scope.stopChecking();
				});

				$scope.stopChecking = function() {
					if($scope.checkInterval === undefined) return;
					$interval.cancel($scope.checkInterval);
					$scope.checkInterval = undefined;
				};

				$scope.viewLogs = function(job) {
					var logsFilter = filter.new("logs_filter");
					logsFilter.setFilter(1);

					var logsSubFilter = filter.new("logs_subfilter");
					logsSubFilter.setFilter(job._id);

					$scope.changeView('/logs');
				};

				$scope.duplicate = function(job) {

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This backup job will be duplicated!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.duplicate = true;

							jetapi.duplicateBackupJob({ _id: job._id }, function (response) {

								$scope.loaders.duplicate = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Backup Job duplication cancelled"));
						}

					});

				};

				$scope.toggleState = function(job) {

					$scope.loaders.state = true;

					var newState = !job.disabled;

					jetapi.manageBackupJob({ action: 'modify', _id: job._id, disabled: newState }, function(response) {

						$scope.loaders.state = false;

						if(response.success)
						{
							job.disabled = newState;
							SweetAlert.success($scope.LANG.t("Success!"), response.message);
							return;
						}

						SweetAlert.error($scope.LANG.t("Failed!"), response.message);
					});
				};

				$scope.toggleBackupQuota = function (job) {

					$scope.loaders.quota = true;

					var newState = !job.backupquota;

					jetapi.manageBackupJob({ action: 'modify', _id: job._id, backupquota: newState }, function(response) {

						$scope.loaders.quota = false;

						if(response.success)
						{
							job.backupquota = newState;
							SweetAlert.success($scope.LANG.t("Success!"), response.message);
							return;
						}

						SweetAlert.error($scope.LANG.t("Failed!"), response.message);
					});
				};

				$scope.toggleNextRun = function(job) {

					$scope.loaders.nextrun = true;

					if(job.next_run_int && (job.next_run_int - 300) < $scope.currentTime)
					{
						jetapi.stopBackupJob({ _id: job._id }, function(response) {

							$scope.loaders.nextrun = false;

							if(response.success)
							{
								job.next_run = response.data.next_run;
								job.next_run_int = job.next_run ? parseInt((new Date(job.next_run)).getTime() / 1000) : 0;
								SweetAlert.success($scope.LANG.t("Success!"), response.message);
								return;
							}

							SweetAlert.error($scope.LANG.t("Failed!"), response.message);
						});
					}
					else
					{
						jetapi.setBackupJobNextRun({ _id: job._id }, function(response) {

							$scope.loaders.nextrun = false;

							if(response.success)
							{
								job.next_run = response.data.next_run;
								job.next_run_int = job.next_run ? parseInt((new Date(job.next_run)).getTime() / 1000) : 0;
								SweetAlert.success($scope.LANG.t("Success!"), response.message);
								return;
							}

							SweetAlert.error($scope.LANG.t("Failed!"), response.message);
						});
					}
				};

				$scope.recalculating = {};
				$scope.recalculateRating = function (job) {

					if($scope.recalculating[job._id]) return;
					$scope.recalculating[job._id] = true;

					jetapi.calculateBackupJobRating({ _id: job._id }, function(response) {

						$scope.recalculating[job._id] = false;

						if(response.success)
						{
							job.rating = response.data.rating;
							SweetAlert.success($scope.LANG.t("Success!"), response.message);
							return;
						}

						SweetAlert.error($scope.LANG.t("Failed!"), response.message);
					});

				};

				$scope.onClickDelete = function(job) {

					if($scope.saveing) return;
					$scope.saveing = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This backup job will be permanently deleted!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteBackupJob({ _id: job._id }, function(response) {

								$scope.saveing = false;
								$scope.loaders.delete = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Backup Job delete cancelled"));
						}

					});
				};

				jetapi.listAccountFilters({}, function (response) {
					if(response.success) {
						for(var i = 0; i < response.data.filters.length; i++) {
							$scope.filters[response.data.filters[i].group_id] = response.data.filters[i].name;
						}
					}
					$scope.$on('fetch', $scope.fetchData);
				});

				$scope.createBackupJob = function() {
					$scope.destinationsCheck(function () {
						$scope.changeView('/backupJobManage');
					});
				};

				$scope.startWizard = function() {
					$scope.wizards.backupJobs($scope.fetchData);
				};

				$scope.destinationsCheck = function (callback) {
					if(callback === undefined) callback = function () {};

					if($scope.destinations !== undefined) {

						if(!$scope.destinations) {
							SweetAlert.error($scope.LANG.t("Error!"), $scope.LANG.t("You must first create a destination before creating a backup job for this server"));
							return;
						}

						callback();
						return;
					}

					$scope.destinations = false;

					jetapi.listDestinations({}, function(response) {

						for(var i = 0; i < response.data.destinations.length; i++) {
							var destination = response.data.destinations[i];
							if(destination.backup_destination && destination.engine == CONSTS.DESTINATION_ENGINE_JETBACKUP) {
								$scope.destinations = true;
								break;
							}
						}


						if(!$scope.destinations) {
							SweetAlert.error($scope.LANG.t("Error!"), $scope.LANG.t("You must first create a destination before creating a backup job for this server"));
							return;
						}

						callback();
					});
				};


				$scope.fetchData = function() {

					$scope.loadingJobs = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.jobs = [];

					jetapi.listBackupJobs(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.jobs);
						$scope.jobs = response.data.jobs;
						for(var i = 0; i < $scope.jobs.length; i++) {
							$scope.jobs[i].filters_sorted = [];

							for(var j = 0; j < $scope.jobs[i].filters.length; j++) {


								for(var a = 0; a < $scope.jobs[i].filters[j].length; a++) {
									$scope.jobs[i].filters_sorted.push($scope.jobs[i].filters[j][a]);
								}
							}

							$scope.jobs[i].next_run_int = $scope.jobs[i].next_run ? parseInt((new Date($scope.jobs[i].next_run)).getTime() / 1000) : 0;
						}
						$scope.loadingJobs = false;
					});
				};

				$timeout($scope.fetchData);
			}
		]
	);
});