'use strict';

define([
	'app',
	'libraries/utils',
	'libraries/permissions',
	'libraries/consts'
], function(app, UTILS, PERMISSIONS, CONSTS) {
	app.controller("backupManagerAccounts",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert", "lang", "storage",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert, lang, storage) {

				// $scope.orphan = 0;
				$scope.accounts = [];
				$scope.backupDifferences = {};
				$scope.loadingAccounts = false;
				$scope.status = undefined; //Used to display errors.
				$scope.hasChecked = false;

				// init models
				$scope.models = {};
				$scope.models.backupType = CONSTS.BACKUP_TYPE_FULL;
				$scope.models.destination = '';
				$scope.models.structure = 0;
				$scope.models.orphan = 0;

				meta = meta.new("backup_manager_accounts");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				meta.setSortReverse(false);
				if(!meta.getSortBy()) meta.setSortBy("username");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["username", "disk_space_usage", "disk_files_usage", "partition", "owner"]);
				meta.setTotalItems($scope.accounts.length);


				/* Permissions handlers */
				$scope.canView = {
					1: PERMISSIONS.canViewConfigBackups,
					2: PERMISSIONS.canViewFileBackups,
					4: PERMISSIONS.canViewDatabaseBackups,
					8: PERMISSIONS.canViewEmailBackups,
					16: PERMISSIONS.canViewCronBackups,
					32: PERMISSIONS.canViewDNSZoneBackups,
					127: PERMISSIONS.canViewFullBackups
				};

				$scope.canRestore = {
					1: PERMISSIONS.canRestoreConfigBackups,
					// 2: PERMISSIONS.canRestoreFileBackups,
					4: PERMISSIONS.canRestoreDatabaseBackups,
					8: PERMISSIONS.canRestoreEmailBackups,
					16: PERMISSIONS.canRestoreCronBackups,
					32: PERMISSIONS.canRestoreDNSZoneBackups,
					127: PERMISSIONS.canRestoreFullBackups
				};

				$scope.canDownload = {
					1: PERMISSIONS.canDownloadConfigBackups,
					2: PERMISSIONS.canDownloadFileBackups,
					4: PERMISSIONS.canDownloadDatabaseBackups,
					8: PERMISSIONS.canDownloadEmailBackups,
					16: PERMISSIONS.canDownloadCronBackups,
					32: PERMISSIONS.canDownloadDNSZoneBackups,
					127: PERMISSIONS.canDownloadFullBackups
				};

				$scope.hasRestorePermissions = function () {
					return PERMISSIONS.canViewDestinations && PERMISSIONS.canRestoreFullBackups && PERMISSIONS.canRestoreFileBackups &&
						PERMISSIONS.canRestoreEmailBackups && PERMISSIONS.canRestoreDatabaseBackups && PERMISSIONS.canRestoreDNSZoneBackups;
				};
				/* END Permissions handlers */

				/**Dropdown sections*/
				$scope.destinations = [{_id: '', name: $scope.LANG.t('All') }];

				$scope.flags = [
					{ label: $scope.LANG.t("All"), value: 0 },
					{ label: $scope.LANG.t("Incremental"), value: 1 },
					{ label: $scope.LANG.t("Archived"), value: 2 },
					{ label: $scope.LANG.t("Compressed"), value: 4 }
				];

				$scope.backupTypes = [];
				var backupTypes = [
					{ label: $scope.LANG.t("Full Backups"), value: CONSTS.BACKUP_TYPE_FULL, permission: $scope.canView[127] },
					{ label: $scope.LANG.t("File Backups"), value: CONSTS.BACKUP_TYPE_FILES, permission: !$scope.disasterRecovery && $scope.canView[2] },
					{ label: $scope.LANG.t("Database Backups"), value: CONSTS.BACKUP_TYPE_DATABASES, permission: !$scope.disasterRecovery && $scope.canView[4] },
					{ label: $scope.LANG.t("Email Backups"), value: CONSTS.BACKUP_TYPE_EMAILS, permission: !$scope.disasterRecovery && $scope.canView[8] },
					{ label: $scope.LANG.t("Cron Job Backups"), value: CONSTS.BACKUP_TYPE_CRON_JOBS, permission: !$scope.disasterRecovery && $scope.canView[16] },
					{ label: $scope.LANG.t("DNS Zone Backups"), value: CONSTS.BACKUP_TYPE_DNS_ZONES, permission: !$scope.disasterRecovery && $scope.canView[32] }
				];

				for(var i = 0; i < backupTypes.length; i++) {
					if(backupTypes[i].permission) $scope.backupTypes.push(backupTypes[i]);
				}
				/**END Dropdown sections*/

				//Displayed backup type names
				$scope.account_types_names = {
					1: null,
					2: null,
					4: $scope.LANG.t('Database'),
					8: $scope.LANG.t('Email'),
					16: null,
					32: $scope.LANG.t('Domain'),
					127: null
				};

				$scope.$watch('models.backupType', function() {
					if(!$scope.models.orphan) $scope.models.structure = CONSTS.BACKUP_STRUCTURE_INCREMENTAL;
					$scope.fetchData();
				});
				$scope.$watch('models.destination', function() { $scope.fetchData(); });
				$scope.$watch('models.structure', 	function() { $scope.fetchData(); });

				$scope.selectBackup = function (account) {
					$scope.model = $uibModal.open({
						animation: true,
						size: "lg",
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/backupSelect.ptt',
						controller: 'backupSelect',
						resolve: {
							account: function () { return account; },
							backupType: function () { return $scope.models.backupType; },
							structure: function () { return $scope.models.structure; },
							destination: function () { return $scope.models.destination; }
						}
					});
					$scope.model.result.then(function(result){
						if(!result.backup) return;
						for(var i=0; i<$scope.accounts.length; i++) {
							if($scope.accounts[i].username === result.account.username) {
								handleBackupDifferences($scope.accounts[i], result.backup);
								$scope.accounts[i].backup = result.backup;
								return;
							}
						}
					},function () {
					});
				};

				// This function was build for case that all accounts records checked to get the differences list from origin list.
				var handleBackupDifferences = function(account, newBackup){
					if(account.backup._id === newBackup._id) return;

					if($scope.backupDifferences[account.username] && ($scope.backupDifferences[account.username].origin._id === newBackup._id))
						delete $scope.backupDifferences[account.username];
					else if($scope.backupDifferences[account.username] && $scope.backupDifferences[account.username].origin) $scope.backupDifferences[account.username].replaced = newBackup;
					else $scope.backupDifferences[account.username] = {origin: account.backup, replaced:newBackup};
				};

				/* NOTES*/
				$scope.editNotes = function (backup) {
					if(!PERMISSIONS.canAddBackupNotes) return;
					backup.editing = true;
				};

				$scope.saveNotes = function (backup, keyEvent) {
					if(keyEvent && keyEvent.which !== 13) return;
					jetapi.manageBackup({ _id: backup._id, notes: backup.notes });
					backup.editing = false;
				};
				/* END NOTES*/

				$scope.calculatingBackupSize = '';
				$scope.calculateBackupSize = function (account) {

					if($scope.calculatingBackupSize) return;
					$scope.calculatingBackupSize = account.backup._id;

					jetapi.calculateBackupSize({ _id: account.backup._id }, function(response) {
						$scope.calculatingBackupSize = '';
						if(!response.success) {
							SweetAlert.error(lang.t("Error!"), lang.t(response.message));
							return;
						}
						account.backup.size = response.data.size;
					});
				};

				var buildAllAccountsQuery = function() {
					var backupFilter = {
						type:$scope.models.backupType,
						exclude:[],
						include:[]
					};
					if($scope.models.destination) backupFilter.destination = $scope.models.destination;
					if($scope.models.structure) backupFilter.backup_flags = $scope.models.structure;
					if($scope.backupDifferences) {
						for (var acct in $scope.backupDifferences) {
							backupFilter.exclude.push(acct);
							backupFilter.include.push($scope.backupDifferences[acct].replaced._id);
						}
					}
					return backupFilter;
				};

				/**
				 * backups - backups array
				 * lock - lock status boolean
				 * ttl - time to lock int
				 */
				var toggleLock = function(accounts, lock, ttl, callback) {
					var ids = [];
					for(var i = 0; i < accounts.length; i++) if(accounts[i].backup) ids.push(accounts[i].backup._id);
					jetapi.manageBackup({ _id: ids, lock_ttl: ttl, locked: lock }, function(response) {
						if(callback !== undefined) callback(response);
					});
				};

				var getCheckedAndToggleLock = function(state, ttl) {
					if ($scope.allPagesAccountsChecked) {
						jetapi.manageBackup({ backup_filter: buildAllAccountsQuery(), lock_ttl: ttl, locked: state }, function(response) {
							handleLockResponse(response, state);
						});
					}
					else {
						toggleLock($scope.getChecked(), state? state:0, ttl? ttl:0, function (response) {
							handleLockResponse(response, 0);
						});
					}
				};

				var handleLockResponse = function(response, state) {
					if(!response.success) {
						SweetAlert.error($scope.LANG.t("Error!"), response.message);
						return;
					}
					var accounts = $scope.getChecked();
					for(var i = 0; i < accounts.length; i++) if(accounts[i].backup) accounts[i].backup.locked = state ? state: 0;
					SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("The selected backups locked successfully"));
				};

				$scope.lockSelected = function() {
					$scope.lockTimeSelection(function (ttl) {
						getCheckedAndToggleLock(1, ttl);
					});
				};

				$scope.unlockSelected = function() {
					getCheckedAndToggleLock(0,0);
				};

				$scope.toggleLock = function(account) {
					var status = !account.backup.locked;

					$scope.lockTimeSelection(function (ttl) {
						toggleLock([account], status, ttl, function (response) {

							if (response.success) {
								account.backup.locked = status;

								if(ttl) {
									var today = new Date();
									today.setDate(today.getDate() + parseInt(ttl));
									account.backup.lock_ttl = today;
								} else {
									account.backup.lock_ttl = '';
								}
								SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t(response.message));
								return;
							}

							SweetAlert.error($scope.LANG.t("Error!"), response.message);
						});
					}, !status);
				};

				$scope.showFileType = function() { return $scope.models.backupType == CONSTS.BACKUP_TYPE_FILES;	};
				$scope.disableStructureChange = function() { return $scope.models.backupType < CONSTS.BACKUP_TYPE_FULL;	};


				$scope.restoreSelected = function() {

					$scope.status = undefined;

					if(!$scope.disasterRecovery) {
						var messages = [];

						for(var i in $scope.config.restore_conditions)	{
							if($scope.config.restore_conditions[i].type !== 0 && $scope.config.restore_conditions[i].type !== $scope.models.backupType) continue;
							messages.push("<strong>" + $scope.config.restore_conditions[i].title + "</strong><br /><span style='font-size: 13px;'>" + $scope.config.restore_conditions[i].content + "</span>");
						}

						var swal = {
							title: lang.t("Are you sure?"),
							icon: "warning",
							buttons: [lang.t("Cancel"), true]
						};

						if(messages.length) {
							var div = document.createElement('div');
							div.classList.add("swal-html-content");
							div.innerHTML = messages.join("<br /><br />");
							swal.content = div;
						} else {
							swal.text = lang.t("The selected accounts will be restored to the selected backup points");
						}

						SweetAlert.swal(swal, function(isConfirm){
							if (isConfirm) accountsRestorePart();
							else SweetAlert.error(lang.t("Cancelled!"), lang.t("Restore cancelled"));
						});
					} else {
						accountsRestorePart();
					}

				};

				var accountsRestorePart = function() {
					var failed = ["<br />"];

					if($scope.allPagesAccountsChecked) {
						startRestoreAll(buildAllAccountsQuery());
						return;
					}
					// else

					var accounts = $scope.getChecked();
					for(var i=0; i< accounts.length; i++) {

						if(!accounts[i].backup) continue;
						$scope.selectedAccount = accounts[i];

						var selected_id = accounts[i].backup._id;
						var encryption_key = accounts[i].backup.encryption_key;
						var backup = accounts[i].backup;

						if(backup.encrypted && !encryption_key) {
							failed.push(lang.t("Account \"%s\"", account) + ": " + lang.t("is encrypted with remote key - Please provide the key "));

							$scope.status = {
								message: failed.join("<br />"),
								type: "danger",
								closeable: false,
								ttl: 10000
							};
							continue;
						}

						var apiParams = { _id: selected_id, files: ['/']};
						if(encryption_key) apiParams.encryption_key = encryption_key;

						jetapi.addQueueRestore(apiParams, function(response, params) {

							params.account.checked = false;
							if(response.success) {
								params.account.backup.queue.restore = response.data._id;
							} else {
								failed.push(lang.t("Account \"%s\"", params.account) + ": " + lang.t(response.message));

								$scope.status = {
									message: failed.join("<br />"),
									type: "danger",
									closeable: false,
									ttl: 5000
								};
							}
						}, {account: accounts[i]});

					}

					$scope.checkall = false;
					$scope.checked = false;
					if(!$scope.disasterRecovery) {
						SweetAlert.swal({
							title: lang.t("Success"),
							text: lang.t("Your request to add the selected account(s) to the restore queue finished successfully. Please check the top of this page for any errors processing this request."),
							icon: "success",
							buttons: [lang.t("Ok"), lang.t("Go to Queue")]
						}, function(goToQueue){
							if(goToQueue) $location.path("/queues");
						});

						//SweetAlert.success(lang.t("Success!"), lang.t("Your request to add the selected account to the restore queue finished successfully. Please check the top of this page to see is there was errors while processing this request."));
					}
				};

				var startRestoreAll = function(backupFilter){
					jetapi.addQueueRestore({backup_filter: backupFilter}, function(response) {
						if(response.success) {
							SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("The selected backups added to the queue - Please note! encrypted backups won't be restored"));
							$scope.clearSelection();
							$scope.fetchData();
						} else {
							$scope.status = {
								message: response.message,
								type: "danger",
								closeable: false,
								ttl: 3000
							};
						}
					});

				};

				$scope.downloadSelected = function () {

					$scope.status = undefined;

					SweetAlert.swal({
						title: lang.t("Are you sure?"),
						text: lang.t("The selected backups will be added to the download queue."),
						icon: "warning",
						buttons: [lang.t("Cancel"), true]
					}, function(isConfirm){
						if (isConfirm) {
							var failed = ["<br />"];

							if($scope.allPagesAccountsChecked) {
								downloadAllByFilter(buildAllAccountsQuery());
								return;
							}

							var accounts = $scope.getChecked();
							for(var i=0; i < accounts.length; i++) {
								if(!accounts[i].backup) continue;
								var selected_id = accounts[i].backup._id;
								var encryption_key = accounts[i].backup.encryption_key;
								var backup = accounts[i].backup;

								if(backup.encrypted && !encryption_key) {
									failed.push(lang.t("Account \"%s\"", account) + ": " + lang.t("You must provide encryption key"));

									$scope.status = {
										message: failed.join("<br />"),
										type: "danger",
										closeable: false,
										ttl: 10000
									};
									continue;
								}
								
								var apiParams = { _id: selected_id, files: ['/'] };
								if(encryption_key) apiParams.encryption_key = encryption_key;


								jetapi.addQueueDownload(apiParams, function(response, params) {
									params.account.backup.checked = false;

									if(response.success && response.data.added) {
										params.account.backup.queue.download = response.data._id;
									} else {
										failed.push(lang.t("Account \"%s\"", params.account.username) + ": " + lang.t(response.message));

										$scope.status = {
											message: failed.join("<br />"),
											type: "danger",
											closeable: false,
											ttl: 10000
										};
									}
								}, {account: accounts[i]});
							}

							$scope.clearSelection();

							SweetAlert.swal({
								title: lang.t("Success"),
								text: lang.t("Your request to add the selected account(s) to the download queue finished successfully. Please check the top of this page for any errors processing this request."),
								icon: "success",
								buttons: [lang.t("Ok"), lang.t("Go to Queue")]
							}, function(goToQueue){
								if(goToQueue) $location.path("/queues");
							});

							//SweetAlert.success(lang.t("Success!"), lang.t("Your request to add the selected account to the download queue finished successfully. Please check the top of this page to see is there was errors while processing this request."));
						}
						else
						{
							SweetAlert.error(lang.t("Cancelled!"), lang.t("Download cancelled"));
						}

					});
				};

				var downloadAllByFilter = function(backupFilter) {
					jetapi.addQueueDownload({backup_filter:backupFilter}, function(response) {

						if(response.success) {
							SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("The selected backups added to the queue - Please note! encrypted backups won't be restored"));
							$scope.clearSelection();
							$scope.fetchData();
						} else {
							$scope.status = {
								message: response.message,
								type: "danger",
								closeable: false,
								ttl: 3000
							};
						}
					});
				};

				$scope.setCheckAllPage = function() {
					if(!$scope.checkall) $scope.allPagesAccountsChecked = false;
					for (var i=0; i < $scope.accounts.length; i++) $scope.accounts[i].checked = $scope.checkall;
				};

				$scope.checkInAllPages = function(){
					$scope.allPagesAccountsChecked = true;
				};

				$scope.getChecked = function() {
					var checkedAccounts = [];
					for(var i=0; i<$scope.accounts.length; i++) if($scope.accounts[i].checked) checkedAccounts.push($scope.accounts[i]);
					return checkedAccounts;
				};

				$scope.clearSelection = function() {
					for (var i=0; i < $scope.accounts.length; i++) $scope.accounts[i].checked = false;
					$scope.allPagesAccountsChecked = false;
					$scope.checkall = false;
				};

				$scope.$watch('accounts', function() {
					var checked = $scope.getChecked();
					if(checked.length < $scope.accounts.length) {
						$scope.allPagesAccountsChecked = false;
						$scope.checkall = false;
					}
					$scope.hasChecked = checked.length;
				},true);

				//TODO - Handler the orphans switch
				$scope.$watch("models.orphan", function(){
					if(!$scope.accounts_loaded) return;
					if($scope.models.orphan) $scope.models.backupType = CONSTS.BACKUP_TYPE_FULL;
					$scope.fetchData();
				}, true);


				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					if($scope.loadingAccounts) return;
					$scope.loadingAccounts = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						filter: meta.getFilter(),
						type:$scope.models.orphan == 1 ? CONSTS.BACKUP_TYPE_FULL : $scope.models.backupType,
						orphan: $scope.models.orphan,
						dr_mode: $scope.disasterRecovery,
					};
					if($scope.models.destination) apiParams.destination = $scope.models.destination;
					if($scope.models.structure) apiParams.backup_flags = $scope.models.structure;

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.accounts = [];

					if(storage.exists("backup_manager_accounts_filter")) {
						var backupMeta = storage.create("backup_manager_accounts_filter");
						var account = backupMeta.get('account');
						apiParams = {find:{username:account}, type:CONSTS.BACKUP_TYPE_FULL};
						storage.delete("backup_manager_accounts_filter");
					}

					jetapi.listAccountsBackups(apiParams, function (response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.accounts);
						$scope.accounts = response.data.accounts;
						for(var i = 0; i < $scope.accounts.length; i++) {
							if($scope.accounts[i].backup) {
								var backup = $scope.accounts[i].backup;
								$scope.accounts[i].backup.display = lang.d(backup.created) + ($scope.account_types_names[$scope.models.backupType] != null ? ' (' + $scope.account_types_names[$scope.models.backupType] + ': ' + backup.name + ')' : '') + (backup.encrypted ? ' - Encrypted' : '');
							}
						}
						$scope.accounts_loaded = true;
						$scope.loadingAccounts = false;
					});
				};

				$scope.fetchDestinations = function() {
					jetapi.listDestinations({}, function(response) {
						if(!response.success) return;

						for(var i = 0; i < response.data.destinations.length; i++) {
							if(response.data.destinations[i].clone_destination) continue;
							$scope.destinations.push({
								_id: response.data.destinations[i]._id,
								name: response.data.destinations[i].name + " (" + response.data.destinations[i].type + ")"
							});
						}
					});
				};

				//TODO - this suppose to clean the search input - Confirm its working
				$scope.$on('$destroy', function() {
					meta.setFilter('');
				});

				$timeout($scope.fetchDestinations());
				$timeout($scope.fetchData());
			}]);
});