'use strict';

define([
	'app',
	'libraries/utils',
	'libraries/permissions'
], function(app, PERMISSIONS) {
	app.controller("backupSelect",
		["$scope", "$uibModalInstance", "jetapi", "lang", "meta", "account", "backupType", "structure", "destination",
			function ($scope, $uibModalInstance, jetapi, lang, meta, account, backupType, structure, destination) {

				$scope.LANG = lang;
				$scope.backups = [];
				$scope.loading = true;
				$scope.account = account;
				$scope.backupType = backupType;

				$scope.types_names = {
					1: null,
					2: null,
					4: $scope.LANG.t('Database'),
					8: $scope.LANG.t('Email'),
					16: null,
					32: $scope.LANG.t('Domain'),
					127: null
				};



				meta = meta.new("account_backups");
				$scope.meta = meta;
				$scope.meta.setPageSizes([5]);
				$scope.meta.setPageSize(5);
				$scope.metaData = meta.getData();
				meta.setSortReverse(false);
				if(!meta.getSortBy()) meta.setSortBy("created");
				if(!meta.getSortDirection()) meta.setSortDirection("dec");
				meta.setSortFields(["created"]);
				meta.setTotalItems($scope.backups.length);



				$scope.fetchData = function(){
					var apiParams = {
						sort: {},
						limit: meta.getPageSize(),
						skip: meta.getSkip(),
						accounts: [account.username],
						type: backupType ? backupType : 127
					};

					if(destination) apiParams.destination = $scope.destination;
					if(structure) apiParams.backup_flags = $scope.structure;

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					jetapi.listBackups(apiParams, function (response) {
						$scope.backups = response.data.backups;
						meta.setTotalItems(response.data.total);
						meta.calculate($scope.backups);
						for(var i=0; i<$scope.backups.length; i++) $scope.backups[i].display = lang.d($scope.backups[i].created) + ($scope.types_names[$scope.backupType] != null ? ' (' + $scope.types_names[$scope.backupType] + ': ' + $scope.backups[i].name + ')' : '') + ($scope.backups[i].encrypted ? ' - Encrypted' : '');

						$scope.loading = false;
					});
				};

				$scope.selectBackup = function(backup) {
					$uibModalInstance.close({account:$scope.account, backup:backup});
				};

				$scope.cancel = function () {
					$uibModalInstance.dismiss($scope.LANG.t('cancel'));
				};

				$scope.fetchData();
	}]);
});