'use strict';

define([
	'app',
	], function(app) {
	app.controller("cloneJobs",
		["$rootScope", "$scope", "$location", "$timeout", "$interval", "$q", "jetapi", "meta", "filter", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, $interval, $q, jetapi, meta, filter, SweetAlert) {
				$rootScope.$emit('menuItem', 'CloneJobs');

				$scope.jobs = [];
				$scope.loadingJobs = false;

				$scope.currentTime = parseInt(Date.now() / 1000);
				$interval(function () { $scope.currentTime = parseInt(Date.now() / 1000); }, 100);

				$scope.loaders = {
					state: false,
					nextrun: false,
					duplicate: false,
					delete: false
				};

				meta = meta.new("clone_jobs");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("domain");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["domain", "ip", "username", "disk_space_usage", "disk_files_usage", "partition", "owner"]);
				meta.setTotalItems($scope.jobs.length);

				$scope.checkProgress = function () {

					for(var i = 0; i < $scope.jobs.length; i++)
					{
						if(!$scope.jobs[i].running) continue;

						(function (i) {
							jetapi.getCloneJob({ _id: $scope.jobs[i]._id }, function(response) {
								// update all job data
								for(var a in response.data) {
									$scope.jobs[i][a] = response.data[a];
									$scope.jobs[i].next_run_int = $scope.jobs[i].next_run ? parseInt((new Date($scope.jobs[i].next_run)).getTime() / 1000) : 0;
								}
							});
						})(i);
					}
				};

				$scope.checkInterval = $interval($scope.checkProgress, 3000);

				$scope.$on("$destroy", function() {
					$scope.stopChecking();
				});

				$scope.stopChecking = function() {
					if($scope.checkInterval === undefined) return;
					$interval.cancel($scope.checkInterval);
					$scope.checkInterval = undefined;
				};

				$scope.viewLogs = function(job) {
					var logsFilter = filter.new("logs_filter");
					logsFilter.setFilter(6);

					var logsSubFilter = filter.new("logs_subfilter");
					logsSubFilter.setFilter(job._id);

					$scope.changeView('/logs');
				};

				$scope.duplicate = function(job) {

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This clone job will be duplicated!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.duplicate = true;

							jetapi.duplicateCloneJob({ _id: job._id }, function (response) {

								$scope.loaders.duplicate = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Clone Job duplication cancelled"));
						}

					});
				};

				$scope.toggleState = function(job) {

					$scope.loaders.state = true;

					var newState = !job.disabled;

					jetapi.manageCloneJob({ action: 'modify', _id: job._id, disabled: newState }, function(response) {

						$scope.loaders.state = false;

						if(response.success)
						{
							job.disabled = newState;
							SweetAlert.success($scope.LANG.t("Success!"), response.message);
							return;
						}

						SweetAlert.error($scope.LANG.t("Failed!"), response.message);
					});
				};

				$scope.toggleNextRun = function(job) {

					$scope.loaders.nextrun = true;

					if(job.next_run_int && (job.next_run_int - 300) < $scope.currentTime)
					{
						jetapi.stopCloneJob({ _id: job._id }, function(response) {

							$scope.loaders.nextrun = false;

							if(response.success)
							{
								job.next_run = response.data.next_run;
								job.next_run_int = job.next_run ? parseInt((new Date(job.next_run)).getTime() / 1000) : 0;
								SweetAlert.success($scope.LANG.t("Success!"), response.message);
								return;
							}

							SweetAlert.error($scope.LANG.t("Failed!"), response.message);
						});
					}
					else
					{
						jetapi.setCloneJobNextRun({ _id: job._id }, function(response) {

							$scope.loaders.nextrun = false;

							if(response.success)
							{
								job.next_run = response.data.next_run;
								job.next_run_int = job.next_run ? parseInt((new Date(job.next_run)).getTime() / 1000) : 0;
								SweetAlert.success($scope.LANG.t("Success!"), response.message);
								return;
							}

							SweetAlert.error($scope.LANG.t("Failed!"), response.message);
						});
					}
				};

				$scope.onClickDelete = function(job) {

					if($scope.saveing) return;
					$scope.saveing = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This clone job will be permanently deleted!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteCloneJob({ _id: job._id }, function(response) {

								$scope.loaders.delete = false;
								$scope.saveing = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Clone Job delete cancelled"));
						}

					});
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingJobs = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						filter: meta.getFilter()
					};

					$scope.jobs = [];

					jetapi.listCloneJobs(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.jobs);
						$scope.jobs = response.data.jobs;
						for(var i in $scope.jobs) $scope.jobs[i].next_run_int = $scope.jobs[i].next_run ? parseInt((new Date($scope.jobs[i].next_run)).getTime() / 1000) : 0;
						$scope.loadingJobs = false;
					});
				};

				$timeout($scope.fetchData);
			}
		]
	);
});