define([
	'app',
], function(app) {
	app.controller("dashboard",
		["$rootScope", "$scope", "$location", "$timeout", "$interval", "jetapi", "meta",
			function($rootScope, $scope, $location, $timeout, $interval, jetapi, meta) {
				$rootScope.$emit('menuItem', 'Dashboard');

				$scope.stats = {
					alerts: 0,
					accounts: 0,
					jobs: 0,
					backups: 0,
					over_quota: 0,
					jobs_running: 0
				};


				$scope.loadingStatistics = true;
				$scope.loadingDestinations = false;
				$scope.loadingBackups = false;
				$scope.loadingClones = false;
				$scope.loadingQueues = false;
				$scope.loadingAlerts = false;

				$scope.restore_queue_summary = {};
				$scope.alerts = [];
				$scope.destinations = [];
				$scope.backups = [];
				$scope.clones = [];
				$scope.queues = [];
				$scope.accounts = [];
				$scope.interval = undefined;

				$scope.destinationMeta = meta.new("dashboard_destinations");
				$scope.destinationMetaData = $scope.destinationMeta.getData();
				$scope.destinationMeta.setSortBy("jobs_count");
				$scope.destinationMeta.setSortDirection("asc");
				$scope.destinationMeta.setTotalItems($scope.destinations.length);
				$scope.destinationMeta.setPageSizes([5,10,25]);
				$scope.destinationMeta.setPageSize(5);
				$scope.destinationMeta.setLimit(5);
				$scope.destinationMeta.setMaxPages(1);

				$scope.backupMeta = meta.new("dashboard_backups");
				$scope.backupMetaData = $scope.backupMeta.getData();
				$scope.backupMeta.setSortBy("next_run");
				$scope.backupMeta.setSortDirection("asc");
				$scope.backupMeta.setTotalItems($scope.backups.length);
				$scope.backupMeta.setPageSizes([5,10,25]);
				$scope.backupMeta.setPageSize(5);
				$scope.backupMeta.setLimit(5);
				$scope.backupMeta.setMaxPages(1);

				$scope.cloneMeta = meta.new("dashboard_clones");
				$scope.cloneMetaData = $scope.cloneMeta.getData();
				$scope.cloneMeta.setSortBy("next_run");
				$scope.cloneMeta.setSortDirection("asc");
				$scope.cloneMeta.setTotalItems($scope.clones.length);
				$scope.cloneMeta.setPageSizes([5,10,25]);
				$scope.cloneMeta.setPageSize(5);
				$scope.cloneMeta.setLimit(5);
				$scope.cloneMeta.setMaxPages(1);

				$scope.queueMeta = meta.new("dashboard_queues");
				$scope.queueMetaData = $scope.queueMeta.getData();
				$scope.queueMeta.setSortBy("created");
				$scope.queueMeta.setSortDirection("desc");
				$scope.queueMeta.setTotalItems($scope.queues.length);
				$scope.queueMeta.setPageSizes([5,10,25]);
				$scope.queueMeta.setPageSize(5);
				$scope.queueMeta.setLimit(5);

				$scope.accountMeta = meta.new("dashboard_accounts_without_backupjob");
				$scope.accountMetaData = $scope.accountMeta.getData();
				$scope.accountMeta.setSortBy("username");
				$scope.accountMeta.setSortDirection("asc");
				$scope.accountMeta.setTotalItems($scope.accounts.length);
				$scope.accountMeta.setPageSizes([5,10,25]);
				$scope.accountMeta.setPageSize(5);
				$scope.accountMeta.setLimit(5);

				/*
				$scope.chart = {
					labels: ["January", "February", "March", "April", "May", "June", "July"],
					series: ['Series A', 'Series B'],
					colors: ['#fd6b2b'],
					data: [[65, 59, 80, 81, 56, 55, 40], [28, 48, 40, 19, 86, 27, 90]],
					datasetOverride: [{ yAxisID: 'y-axis-1',  color: [ 'rgba(253, 107, 43)' ], borderColor: [ 'rgba(253, 107, 43)' ], backgroundColor: [ 'rgba(253, 107, 43, 0.2)' ] }, { yAxisID: 'y-axis-2' }],
					options: {
						scales: {
							yAxes: [
								{
									id: 'y-axis-1',
									type: 'linear',
									display: true,
									position: 'left'
								},
								{
									id: 'y-axis-2',
									type: 'linear',
									display: true,
									position: 'right'
								}
							]
						}
					}
				};
				*/

				$scope.fetchData = function(callback) {
					jetapi.getDashboardDetails({}, function(response) {
						$scope.stats.alerts = response.data.statistics.total_acknowledged_alerts;
						$scope.stats.accounts = response.data.statistics.total_accounts;
						$scope.stats.jobs = response.data.statistics.total_backup_jobs + response.data.statistics.total_clone_jobs;
						$scope.stats.backups = response.data.statistics.total_backups;
						$scope.stats.disk_usage = response.data.statistics.total_disk_usage;
						$scope.stats.jobs_running = response.data.statistics.total_backup_jobs_running + response.data.statistics.total_clone_jobs_running;
						$scope.alerts = response.data.alerts.alerts;
						$scope.restore_queue_summary = response.data.restore_queue_summary;

						$scope.destinationMeta.setTotalItems(response.data.statistics.total_destinations);
						$scope.destinationMeta.calculate(response.data.destinations.destinations);
						$scope.destinations = response.data.destinations.destinations;

						$scope.backupMeta.setTotalItems(response.data.backup_jobs.total);
						$scope.backupMeta.calculate(response.data.backup_jobs.jobs);
						$scope.backups = response.data.backup_jobs.jobs;

						$scope.cloneMeta.setTotalItems(response.data.clone_jobs.total);
						$scope.cloneMeta.calculate(response.data.clone_jobs.jobs);
						$scope.clones = response.data.clone_jobs.jobs;

						$scope.queueMeta.setTotalItems(response.data.queue.total);
						$scope.queueMeta.calculate(response.data.queue.queue);
						$scope.queues = response.data.queue.queue;

						$scope.accountMeta.setTotalItems(response.data.accounts.total);
						$scope.accountMeta.calculate(response.data.accounts.accounts);
						$scope.accounts = response.data.accounts.accounts;

						if(callback !== undefined) callback(response);
					});
				};

				/*
				$scope.interval = $interval(function () {
					$scope.fetchData();
				}, 10000);

				$scope.$on("$destroy", function() {
					if ($scope.interval === undefined) return;
					$interval.cancel($scope.interval);
					$scope.interval = undefined;
				});
				*/

				$scope.$on('fetch', fetch);

				function fetch() {
					$scope.loadingStatistics = true;
					$scope.fetchData(function() {
						$scope.loadingStatistics = false;
					});
				}

				$scope.listDestinations = function() {

					$scope.loadingDestinations = true;
					$scope.destinations = [];

					var sort = {};
					sort[$scope.destinationMeta.getSortBy()] = $scope.destinationMeta.getSortDirectionInt();

					jetapi.listDestinations({
						sort: sort,
						skip: $scope.destinationMeta.getSkip(),
						limit: $scope.destinationMeta.getLimit()
					}, function(response) {
						$scope.destinationMeta.setTotalItems(response.data.total);
						$scope.destinationMeta.calculate(response.data.destinations);
						$scope.destinations = response.data.destinations;
						$scope.loadingDestinations = false;
					});
				};

				$scope.listBackupJobs = function() {

					$scope.loadingBackups = true;
					$scope.backups = [];

					var sort = {};
					sort[$scope.backupMeta.getSortBy()] = $scope.backupMeta.getSortDirectionInt();

					jetapi.listBackupJobs({
						sort: sort,
						skip: $scope.backupMeta.getSkip(),
						limit: $scope.backupMeta.getLimit()
					}, function(response) {
						$scope.backupMeta.setTotalItems(response.data.total);
						$scope.backupMeta.calculate(response.data.jobs);
						$scope.backups = response.data.jobs;
						$scope.loadingBackups = false;
					});
				};

				$scope.listCloneJobs = function() {

					$scope.loadingClones = true;
					$scope.clones = [];

					var sort = {};
					sort[$scope.cloneMeta.getSortBy()] = $scope.cloneMeta.getSortDirectionInt();

					jetapi.listCloneJobs({
						sort: sort,
						skip: $scope.cloneMeta.getSkip(),
						limit: $scope.cloneMeta.getLimit()
					}, function(response) {
						$scope.cloneMeta.setTotalItems(response.data.total);
						$scope.cloneMeta.calculate(response.data.jobs);
						$scope.clones = response.data.jobs;
						$scope.loadingClones = false;
					});
				};

				$scope.listQueueItems = function() {

					$scope.loadingQueues = true;
					$scope.queues = [];

					var sort = {};
					sort[$scope.queueMeta.getSortBy()] = $scope.queueMeta.getSortDirectionInt();

					jetapi.listQueueItems({
						sort: sort,
						skip: $scope.queueMeta.getSkip(),
						limit: $scope.queueMeta.getLimit()
					}, function(response) {
						$scope.queueMeta.setTotalItems(response.data.total);
						$scope.queueMeta.calculate(response.data.queue);
						$scope.queues = response.data.queue;
						$scope.loadingQueues = false;
					});
				};

				$timeout(fetch);
			}
		]
	);
});