'use strict';

define([
	'app',
	], function(app) {
	app.controller("filePermissions",
		["$rootScope", "$scope", "$location", "$timeout", "$interval", "$q", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, $interval, $q, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'Settings');

				$scope.permissions = [];
				$scope.loadingPermissions = false;
				$scope.loaders = {
					delete: false
				};

				meta = meta.new("file_permissions");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("regex");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["regex", "owner", "group", "permissions"]);
				meta.setTotalItems($scope.permissions.length);

				$scope.onClickDelete = function(permission) {

					if($scope.saveing) return;
					$scope.saveing = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This file permission will be permanently deleted!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteFilePermission({ _id: permission._id }, function(response) {

								$scope.loaders.delete = false;
								$scope.saveing = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("File Permission delete cancelled"));
						}

					});
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingPermissions = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.permissions = [];

					jetapi.listFilePermissions(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.permissions);
						$scope.permissions = response.data.permissions;
						$scope.loadingPermissions = false;
					});
				};

				$timeout($scope.fetchData);
			}
		]
	);
});