'use strict';

define([
	'app',
	'libraries/consts'
	], function(app, CONSTS) {
	app.controller("hooks",
		["$rootScope", "$scope", "$location", "$timeout", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'Hooks');

				$scope.hooks = [];
				$scope.loading = false;
				$scope.saveing = false;
				$scope.status = undefined;
				$scope.filter = '';
				$scope.filterOptions = [
					{ label: $scope.LANG.t('All Hook Positions'), value: '' },
					{ label: $scope.LANG.t('Pre'), value: CONSTS.HOOK_TYPE_POSITION_PRE },
					{ label: $scope.LANG.t('Post'), value: CONSTS.HOOK_TYPE_POSITION_POST }
				];

				$scope.loaders = {
					delete: false,
					state: false
				};

				$scope.backups = {};
				$scope.clones = {};
				$scope.destinations = {};

				$scope.types = {};

				$scope.types[CONSTS.BACKUP_TYPE_CONFIG] = $scope.LANG.t("Accounts cPanel Configurations");
				$scope.types[CONSTS.BACKUP_TYPE_FILES] = $scope.LANG.t("Accounts Files");
				$scope.types[CONSTS.BACKUP_TYPE_DATABASES] = $scope.LANG.t("Accounts Databases");
				$scope.types[CONSTS.BACKUP_TYPE_EMAILS] = $scope.LANG.t("Accounts Emails");
				$scope.types[CONSTS.BACKUP_TYPE_CRON_JOBS] = $scope.LANG.t("Accounts Cron Jobs");
				$scope.types[CONSTS.BACKUP_TYPE_DNS_ZONES] = $scope.LANG.t("Accounts DNS Zones");
				$scope.types[CONSTS.BACKUP_TYPE_SSL_CERTIFICATES] = $scope.LANG.t("Accounts SSL Certificates");
				$scope.types[CONSTS.BACKUP_TYPE_FULL] = $scope.LANG.t("Accounts Full");
				$scope.types[CONSTS.BACKUP_TYPE_DIRECTORIES] = $scope.LANG.t("Directories");

				$scope.positions = {};
				$scope.positions[CONSTS.HOOK_POSITION_BACKUP] = $scope.LANG.t("Backup");
				$scope.positions[CONSTS.HOOK_POSITION_BACKUP_ACCOUNT] = $scope.LANG.t("Backup Account");
				$scope.positions[CONSTS.HOOK_POSITION_CLONE] = $scope.LANG.t("Clone");
				$scope.positions[CONSTS.HOOK_POSITION_CLONE_ACCOUNT] = $scope.LANG.t("Clone Account");
				$scope.positions[CONSTS.HOOK_POSITION_RESTORE] = $scope.LANG.t("Restore");
				$scope.positions[CONSTS.HOOK_POSITION_DOWNLOAD] = $scope.LANG.t("Download");
				$scope.positions[CONSTS.HOOK_POSITION_REINDEX] = $scope.LANG.t("Reindex");
				$scope.positions[CONSTS.HOOK_POSITION_SNAPSHOT] = $scope.LANG.t("Snapshot");
				$scope.positions[CONSTS.HOOK_POSITION_DAILY_CRON] = $scope.LANG.t("Daily Cron");

				$scope.position_types = {};
				$scope.position_types[CONSTS.HOOK_TYPE_POSITION_PRE] = $scope.LANG.t("Pre");
				$scope.position_types[CONSTS.HOOK_TYPE_POSITION_POST] = $scope.LANG.t("Post");

				meta = meta.new("hooks");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["name","position_type","position","owner","disabled"]);
				meta.setTotalItems($scope.hooks.length);

				$scope.onClickDelete = function(hook) {

					if($scope.saveing) return;

					$scope.saveing = true;
					$scope.loaders.delete = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This hook will be permanently deleted!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							jetapi.deleteHook({ _id: hook._id }, function(response) {

								$scope.saveing = false;
								$scope.loaders.delete = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							$scope.loaders.delete = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Hook delete cancelled"));
						}

					});
				};

				$scope.toggleStatus = function(hook) {

					$scope.loaders.state = true;

					var disabled = !hook.disabled;

					jetapi.manageHook({
						_id: hook._id,
						action: 'modify',
						disabled: disabled
					}, function(response) {
						$scope.loaders.state = false;
						if(response.success) hook.disabled = disabled;
					});
				};

				$scope.init = false;
				$scope.fetchData = function(init) {

					if(!$scope.init && !init) return;
					$scope.init = true;

					$scope.loading = true;
					$scope.status = undefined;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: { position_type: $scope.filter },
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.hooks = [];

					jetapi.listHooks(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.hooks);

						for(var i = 0; i < response.data.hooks.length; i++) {

							var list = response.data.hooks[i].list;

							if(list.length) {
								switch(response.data.hooks[i].position) {
									case CONSTS.HOOK_POSITION_BACKUP:
									case CONSTS.HOOK_POSITION_BACKUP_ACCOUNT:
										for(var j = 0; j < list.length; j++) list[j] = ($scope.backups[list[j]] !== undefined) ? $scope.backups[list[j]].name : 'Unknown';
										break;

									case CONSTS.HOOK_POSITION_CLONE:
									case CONSTS.HOOK_POSITION_CLONE_ACCOUNT:
										for(var j = 0; j < list.length; j++) list[j] = ($scope.clones[list[j]] !== undefined) ? $scope.clones[list[j]].name : 'Unknown';
										break;

									case CONSTS.HOOK_POSITION_RESTORE:
									case CONSTS.HOOK_POSITION_DOWNLOAD:
										for(var j = 0; j < list.length; j++) list[j] = ($scope.types[list[j]] !== undefined) ? $scope.types[list[j]] : 'Unknown';
										break;

									case CONSTS.HOOK_POSITION_REINDEX:
										for(var j = 0; j < list.length; j++) list[j] = ($scope.destinations[list[j]] !== undefined) ? $scope.destinations[list[j]].name : 'Unknown';
										break;
								}
							}

							$scope.hooks.push(response.data.hooks[i]);

						}

						$scope.loading = false;
					});
				};

				$scope.$on('fetch', $scope.fetchData());

				jetapi.listBackupJobs({}, function(response) {
					for(var i = 0; i < response.data.jobs.length; i++) $scope.backups[response.data.jobs[i]._id] = response.data.jobs[i];

					jetapi.listCloneJobs({}, function(response) {
						for(var i = 0; i < response.data.jobs.length; i++) $scope.clones[response.data.jobs[i]._id] = response.data.jobs[i];

						jetapi.listDestinations({}, function(response) {
							for(var i = 0; i < response.data.destinations.length; i++) $scope.destinations[response.data.destinations[i]._id] = response.data.destinations[i];

							$timeout($scope.fetchData(true));
						});
					});
				});

				$scope.fetch = $scope.fetchData;
			}
		]
	);
});