'use strict';

define([
	'app',
	'libraries/permissions'
	], function(app, PERMISSIONS) {
	app.controller("permissions",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$q", "jetapi", "meta", "SweetAlert", "utils", "storage",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $q, jetapi, meta, SweetAlert, utils, storage) {
				$rootScope.$emit('menuItem', 'Permissions');

				$scope.saveing = false;
				$scope.status = undefined;
				$scope.changed = false;
				$scope.loading = false;
				$scope.modifyDisabled = false;
				$scope.totalFiltered = 0;
				$scope.selectedAccount = '';
				$scope.currentSection = 'whm';

				$scope.details = {};
				$scope.filtered = {};
				$scope.saveData = {};

				$scope.meta = {
					sectionValue: -1,
					filterType: 1,
					filterValue: ""
				};

				$scope.permissions = {
					canView: false,
					canManage: false
				};

				$scope.sections = [];
				var sections = [
					{ _id: 'whm', name: $scope.LANG.t('Permissions'), icon: 'cplogos whm', permission: (PERMISSIONS.canViewWHMPermissions || PERMISSIONS.canViewWHMUserPermissions) },
					{ _id: 'cpanel', name: $scope.LANG.t('Permissions'), icon: 'cplogos cpanel', permission: (PERMISSIONS.canViewCpanelPermissions || PERMISSIONS.canViewCpanelUserPermissions) }
				];

				for(var i = 0; i < sections.length; i++) {
					if(sections[i].permission) $scope.sections.push(sections[i]);
					else if(i === 0) $scope.currentSection = 'cpanel';
				}

				$scope.setSectionPermissions = function() {

					switch($scope.currentSection) {
						case 'whm':
							switch($scope.meta.filterType) {
								case 1:
									$scope.permissions.canView = PERMISSIONS.canViewWHMPermissions;
									$scope.permissions.canManage = PERMISSIONS.canManageWHMPermissions;
								break;
								case 2:
									$scope.permissions.canView = PERMISSIONS.canViewWHMUserPermissions;
									$scope.permissions.canManage = PERMISSIONS.canManageWHMUserPermissions;
								break;
							}
						break;

						case 'cpanel':
							switch($scope.meta.filterType) {
								case 1:
									$scope.permissions.canView = PERMISSIONS.canViewCpanelPermissions;
									$scope.permissions.canManage = PERMISSIONS.canManageCpanelPermissions;
								break;
								case 2:
									$scope.permissions.canView = PERMISSIONS.canViewCpanelUserPermissions;
									$scope.permissions.canManage = PERMISSIONS.canManageCpanelUserPermissions;
								break;
							}
						break;
					}
				};

				$scope.$watch("currentSection", $scope.setSectionPermissions);
				$scope.$watch("meta.filterType", $scope.setSectionPermissions);

				$scope.types = [];
				var types = [
					{ value: 1, label: $scope.LANG.t('Global Permissions'), permission: ($scope.currentSection == 'whm' && PERMISSIONS.canViewWHMPermissions) || ($scope.currentSection == 'cpanel' && PERMISSIONS.canViewCpanelPermissions) },
					{ value: 2, label: $scope.LANG.t('Specific Permissions'), permission: ($scope.currentSection == 'whm' && PERMISSIONS.canViewWHMUserPermissions) || ($scope.currentSection == 'cpanel' && PERMISSIONS.canViewCpanelUserPermissions) }
				];

				for(var i = 0; i < types.length; i++) {
					if(types[i].permission) $scope.types.push(types[i]);
					else if(i === 0) $scope.meta.filterType = 2;
				}

				$scope.filterOptions = {
					whm: [
						{ label: $scope.LANG.t('All Permissions'), value: -1 },
						{ label: $scope.LANG.t('All Privileges'), value: 2 },
						{ label: $scope.LANG.t('Accounts Permissions'), value: 0 },
						{ label: $scope.LANG.t('Backups Permissions'), value: 3 },
						{ label: $scope.LANG.t('Destinations Permissions'), value: 45 },
						{ label: $scope.LANG.t('Backup Jobs Permissions'), value: 27 },
						{ label: $scope.LANG.t('Clone Jobs Permissions'), value: 41 },
						{ label: $scope.LANG.t('Account Filters Permissions'), value: 25 },
						{ label: $scope.LANG.t('Snapshots Permissions'), value: 29 },
						{ label: $scope.LANG.t('Alerts Permissions'), value: 102 },
						{ label: $scope.LANG.t('Hooks Permissions'), value: 49 },
						{ label: $scope.LANG.t('Queues Permissions'), value: 114 },
						{ label: $scope.LANG.t('Logs Permissions'), value: 62 },
						{ label: $scope.LANG.t('Permissions Permissions'), value: 53 }
						//{ label: $scope.LANG.t('Settings Permissions'), value: 79 }
					],
					cpanel: [
						{ label: $scope.LANG.t('All Permissions'), value: -1 },
						{ label: $scope.LANG.t('Backups Permissions'), value: 1 },
						{ label: $scope.LANG.t('Snapshots Permissions'), value: 27 },
						//{ label: $scope.LANG.t('Queues Permissions'), value: 31 },
						{ label: $scope.LANG.t('Settings Permissions'), value: 29 }
					]
				};

				$scope.fetch = function() {

					if(storage.exists("view_account_permissions")) {
						var permissionsMeta = storage.create("view_account_permissions");
						$scope.selectedAccount = permissionsMeta.get('account');
						if ($scope.selectedAccount) $scope.meta.filterType = 2;
						$scope.searchAccountText = $scope.selectedAccount;
						storage.delete("view_account_permissions");
					}

					if($scope.meta.filterType === 2 && !$scope.selectedAccount) {
						$scope.details = {};
						$scope.filtered = {};
						$scope.saveData = {};
						$scope.totalFiltered = 0;
						return;
					}

					$scope.loading = true;

					var apiParams = {};
					apiParams.section = $scope.currentSection;
					if($scope.meta.filterType === 2 && $scope.selectedAccount) apiParams.username = $scope.selectedAccount;

					jetapi.getPermissions(apiParams, function(response) {
						$scope.details = response.data.permissions;
						$scope.totalFiltered = 1;
						$scope.filtered = utils.duplicateObject($scope.details);
						$scope.saveData = utils.duplicateObject($scope.details);
						$scope.loading = false;
					});
				};

				$scope.startFetching = function () {
					$scope.details = {};
					$scope.filtered = {};
					$scope.saveData = {};
					$scope.totalFiltered = 0;
					$scope.fetch();
				};

				$scope.resetPermissions = function () {
					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: ($scope.meta.filterType === 2 && $scope.selectedAccount) ? $scope.LANG.t('Are you sure you want to reset all "%s" permissions for the account "%s" to defaults?', $scope.currentSection, $scope.selectedAccount) : $scope.LANG.t("Are you sure you want to reset all \"%s\" permissions to defaults?", $scope.currentSection),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), $scope.LANG.t("Reset Permissions")]
					}, function(isConfirm){
						if (isConfirm) {

							var apiParams = {};
							apiParams.section = $scope.currentSection;
							if($scope.meta.filterType === 2 && $scope.selectedAccount) apiParams.username = $scope.selectedAccount;

							jetapi.resetPermissions(apiParams, function(response) {

								if(response.success) {
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
									$scope.startFetching();
									return;
								}

								SweetAlert.error($scope.LANG.t("Error!"), response.message);
							});

						}
					});
				};

				$scope.selectAccount = function(account) {
					if(account === undefined) return;
					$scope.selectedAccount = account.username;
					$scope.startFetching();
				};

				$scope.searchAccounts = function(query) {

					if(query)
					{
						var reseller = $scope.currentSection === 'whm';
						var deferred = $q.defer();

						jetapi.listAccounts({ filter: query, sort: { username: 1 } }, function(response) {

							var results = [];
							for(var i = 0; i < response.data.accounts.length; i++) {

								var account = response.data.accounts[i];
								if(reseller && !account.reseller) continue;
								results.push(account);
							}
							deferred.resolve( results );
						});

						return deferred.promise;
					}

					return [];
				};

				$scope.filteredList = function() {

					$scope.filtered = {};
					$scope.totalFiltered = 0;

					if(!$scope.meta.filterValue && $scope.meta.sectionValue === -1) {
						$scope.totalFiltered = 1;
						$scope.filtered = utils.duplicateObject($scope.details);
						return;
					}

					for(var i in $scope.details) {
						var details = PERMISSIONS._get($scope.currentSection, i);
						if(details === null) continue;

						if(
							($scope.meta.filterValue && !(new RegExp('(' + $scope.meta.filterValue + ')', 'gi')).test($scope.LANG.t(details.name))) ||
							($scope.meta.sectionValue >= 0 && $scope.meta.sectionValue != details.section)
						) continue;

						$scope.totalFiltered++;
						$scope.filtered[i] = $scope.details[i];
					}
				};

				$scope.getSection = function (section_id) {

					if(!$scope.sections.length) {
						$location.path("/");
						return null;
					}

					for(var i = 0; i < $scope.sections.length; i++) {
						if($scope.sections[i]._id === section_id) {
							return $scope.sections[i];
						}
					}

					return $scope.sections[0];
				};

				$scope.changeSection = function (section) {

					if($scope.changed)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges(function () {
									$location.path('/permissions/' + section._id)
								});
							} else $location.path('/permissions/' + section._id);
						});
					}
					else $location.path('/permissions/' + section._id);
				};

				$scope.$on('$destroy', function() {
					if($scope.changed)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.loadSection = function (section) {
					if(!section) return;
					$scope.currentSection = section._id;
					$scope.startFetching();
				};

				$scope.loadSection($scope.getSection($routeParams.section ? $routeParams.section : 'whm'));

				$scope.$watch("searchAccountText", function() {
					if($scope.searchAccountText !== '') return;
					$scope.selectAccount({ username: '' });
				});

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details);
				}, true);

				$scope.$watch("currentSection", function(){
					if($scope.loadingGlobal) return;

					$scope.loadingGlobal = true;
					if(!$scope.currentSection) $scope.currentSection = 'whm';
					jetapi.getPermissions({section: $scope.currentSection} , function (response) {
						if(!response.success) return;
						$scope.globalPermissions  = response.data.permissions;
						$scope.loadingGlobal = false;
					});
				});

				$scope.shouldDisable = function (key) {
					return $scope.meta.filterType == 2 && $scope.globalPermissions[key] == 2 && !$scope.modifyDisabled;
				};

				$scope.showDisabledMessage = function() {
					if(!$scope.modifyDisabled) return;
					SweetAlert.warning($scope.LANG.t("Attention!"), $scope.LANG.t("If the global permission is to \"Never\" the specific permission will never be applied!"));
				};

				$scope.saveChanges = function(callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = {
						section: $scope.currentSection,
						permissions: utils.saveParams($scope.saveData, $scope.details)
					};

					if($scope.meta.filterType === 2 && $scope.selectedAccount) apiParams.username = $scope.selectedAccount;

					jetapi.managePermissions(apiParams, function(response) {

						$scope.saveing = false;

						if(response.success)
						{
							$scope.changed = false;
							$scope.details = utils.duplicateObject($scope.saveData);

							for(var i in apiParams.permissions) {
								if($scope.filtered[i] !== undefined) $scope.filtered[i] = apiParams.permissions[i];
							}

							if(callback !== undefined) callback();
							SweetAlert.success($scope.LANG.t("Success!"), response.message);
						}
						else
						{
							$scope.status = {
								message: response.message,
								type: "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

			}]);
});