'use strict';

define([
	'app',
	'libraries/consts',
	], function(app, CONSTS) {
	app.controller("queuePriorityManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'Settings');

				$scope.CONSTS = CONSTS;

				$scope.details = {};
				$scope.saveData = {
					name: '',
					priority: ''
				};

				$scope.saveing 	= false;
				$scope.status 	= undefined;
				$scope.changed 	= false;
				$scope.cancelled = false;


				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.$on('$destroy', function() {
					if($scope.changed && !$scope.cancelled)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/queuePriorities');
				};

				$scope.saveChanges = function(apply) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details);

					apiParams.action = $scope.details._id ? 'modify' : 'create';

					jetapi.manageQueuePriority(apiParams, function(response) {

						$scope.saveing = false;

						if(response.success) {
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);
							if(apply) $location.path('/queuePriorityManage/' + response.data._id);
							else $location.path('/queuePriorities');
							SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
						}
						else
						{
							$scope.status = {
								message: response.message,
								type: response.success ? "success" : "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

				$scope.fetch = function(priority_id) {
					jetapi.getQueuePriority({ '_id': priority_id }, function(response) {
						$scope.details = response.data;
						$scope.saveData = utils.duplicateObject($scope.details);
					});

				};

				var priority_id = $routeParams.id;
				if(priority_id) $timeout($scope.fetch(priority_id));
			}
		]
	);

});