'use strict';

define([
	'app',
	'libraries/permissions',
	'libraries/consts'
	], function(app, PERMISSIONS, CONSTS) {
	app.controller("queues",
		["$rootScope", "$scope", "$location", "$timeout", "$interval", "SweetAlert", "jetapi", "meta",
			function ($rootScope, $scope, $location, $timeout, $interval, SweetAlert, jetapi, meta) {
				$rootScope.$emit('menuItem', 'Queues');

				$scope.queues = {};
				$scope.loadingQueues = false;
				$scope.showPager = true;
				$scope.cancelling = false;
				$scope.interval = 3;
				$scope.filter = 0;
				$scope.checkInterval = undefined;
				$scope.filterOptions = [
					{ label: $scope.LANG.t('All Queues'), value: 0 },
					{ label: $scope.LANG.t('Restore Queues'), value: 1 },
					{ label: $scope.LANG.t('Download Queues'), value: 2 },
					{ label: $scope.LANG.t('Snapshot Queues'), value: 3 },
					{ label: $scope.LANG.t('Clone Queues'), value: 4 }
				];

				$scope.cmdType = {
					1: 'restore',
					2: 'downloads',
					3: 'snapshots',
					4: 'clone'
				};

				$scope.account_types_names = {
					127: null,
					1: null,
					2: null,
					4: $scope.LANG.t('Database'),
					8: $scope.LANG.t('Email'),
					16: null,
					32: $scope.LANG.t('Domain'),
					64: $scope.LANG.t('Domain')
				};

				$scope.canRestore = {
					1: PERMISSIONS.canRestoreConfigBackups,
					2: PERMISSIONS.canRestoreFileBackups,
					4: PERMISSIONS.canRestoreDatabaseBackups,
					8: PERMISSIONS.canRestoreEmailBackups,
					16: PERMISSIONS.canRestoreCronBackups,
					32: PERMISSIONS.canRestoreDNSZoneBackups,
					127: PERMISSIONS.canRestoreFullBackups
				};

				meta = meta.new("queues");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("created");
				if(!meta.getSortDirection()) meta.setSortDirection("desc");
				meta.setSortFields(["created","account","status","type"]);
				meta.setTotalItems($scope.queues.length);


				$scope.redoRestore = function(queue) {
					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("A New restore process will be added to the queue."),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function (isConfirm) {
						if (isConfirm) {
							//jetapi.addQueueRestore({_id: queue.item_id}, function (response) {
							jetapi.queueRetry({_id: queue._id}, function (response) {
								if (response.success) {
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("Queue item successfully added."));
									return;
								}
								SweetAlert.error($scope.LANG.t("Failed!"), response.message);
							});
						}
					});
				};

				$scope.startChecking = function() {
					if(angular.isDefined($scope.checkInterval) || parseInt($scope.interval) <= 0) return;
					$scope.checkInterval = $interval(function () {
						$scope.fetchQueues(function(response) {
							var data = response.data.queue;
							for(var i = 0; i < data.length; i++) {
								if($scope.queues[data[i]._id] === undefined) {
									$scope.fetchData();
									return;
								}
								$scope.queues[data[i]._id] = data[i];
							}
						});
					}, parseInt($scope.interval) * 1000);
				};

				$scope.stopChecking = function() {
					if($scope.checkInterval === undefined) return;
					$interval.cancel($scope.checkInterval);
					$scope.checkInterval = undefined;
				};

				$scope.$on("$destroy", function() {
					$scope.stopChecking();
				});

				$scope.$watch("interval", function(){
					$scope.stopChecking();
					$scope.startChecking();
				}, true);

				$scope.rollback = function(queue) {

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("New restore process from the latest safety backup will be inserted to the queue."),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){
						if (isConfirm) {
							jetapi.addQueueRestore({ _id: queue.safety_id }, function (response) {

								if(response.success)
								{
									$scope.queues[response.data._id] = response.data;
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("Queue item process cancelled successfully"));
									return;
								}

								SweetAlert.error($scope.LANG.t("Failed!"), response.message);
							});
						} else $scope.cancelling = false;
					});

				};

				$scope.cancelQueue = function(queue) {

					if($scope.cancelling) return;

					if(queue.status > 1) {
						SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("This process already started and can't be cancelled"));
						return;
					}

					$scope.cancelling = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("The selected queue item process will be cancelled"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){
						if (isConfirm) {
							jetapi.cancelQueueItem({ _id: queue._id }, function (response) {

								$scope.cancelling = false;

								if(response.success)
								{
									$scope.queues[response.data._id] = response.data;
									SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("Queue item process cancelled successfully"));
									return;
								}

								SweetAlert.error($scope.LANG.t("Failed!"), response.message);
							});
						} else $scope.cancelling = false;
					});
				};

				$scope.queueIcon = function (queue) {
					if(queue.status === 1) return 'fas fa-clock';
					else if(queue.status > 1 && queue.status < 100) return 'fas fa-cog fa-spin';
					else if(queue.status === 100) return 'fas fa-check';
					else if(queue.status > 100) return 'fas fa-times';
					return '';
				};

				$scope.editQueuePriority = function(queue) {
					if(queue.status > CONSTS.QUEUE_STATUS_PENDING) return;
					$scope.stopChecking();
					queue.editing = true;
				};

				$scope.enterSave = function(keyEvent,queue) {
					if (keyEvent.which === 13) $scope.saveNewPriority(queue);
				};

				$scope.saveNewPriority = function(queue){
					jetapi.changeQueueItemPriority({ _id: queue._id, priority: queue.queue_priority}, function (response) {
						queue.editing = false;
						$scope.startChecking();

						if(!response.success) SweetAlert.error($scope.LANG.t("Failed!"), response.message);
					});
				};

				$scope.fetchQueues = function(callback) {

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: { queue_type: $scope.filter },
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					jetapi.listQueueItems(apiParams, function(response) {
						if(callback !== undefined) callback(response);
					});
				};
				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.stopChecking();
					$scope.loadingQueues = true;
					$scope.queues = {};

					$scope.fetchQueues(function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.queue);
						for(var i = 0; i < response.data.queue.length; i++) {
							$scope.queues[response.data.queue[i]._id] = response.data.queue[i];
						}
						$scope.loadingQueues = false;
						$scope.startChecking();
					});
				};

				$timeout($scope.fetchData);

			}
		]
	);
});