'use strict';

define([
	'app',
	], function(app) {
	app.controller("restoreConditions",
		["$rootScope", "$scope", "$location", "$timeout", "$interval", "$q", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, $interval, $q, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'Settings');

				$scope.conditions = [];
				$scope.loadingConditions = false;
				$scope.loaders = {
					state: false,
					delete: false
				};

				meta = meta.new("restore_conditions");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("title");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["title", "content", "type"]);
				meta.setTotalItems($scope.conditions.length);

				$scope.toggleState = function(condition) {

					$scope.loaders.state = true;

					var newState = !condition.disabled;

					jetapi.manageRestoreCondition({ action: 'modify', _id: condition._id, disabled: newState }, function(response) {

						$scope.loaders.state = false;

						if(response.success)
						{
							condition.disabled = newState;
							SweetAlert.success($scope.LANG.t("Success!"), response.message);
							return;
						}

						SweetAlert.error($scope.LANG.t("Failed!"), response.message);
					});
				};

				$scope.onClickDelete = function(condition) {

					if($scope.saveing) return;
					$scope.saveing = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This restore condition will be permanently deleted!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteRestoreCondition({ _id: condition._id }, function(response) {

								$scope.loaders.delete = false;
								$scope.saveing = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Restore Condition delete cancelled"));
						}

					});
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingConditions = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.conditions = [];

					jetapi.listRestoreConditions(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.conditions);
						$scope.conditions = response.data.conditions;
						$scope.loadingConditions = false;
					});
				};

				$timeout($scope.fetchData);
			}
		]
	);
});