'use strict';

define([
	'app',
	], function(app) {
	app.controller("schedules",
		["$rootScope", "$scope", "$location", "$timeout", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, jetapi, meta, SweetAlert) {
				//$rootScope.$emit('menuItem', 'Schedules');

				$scope.schedules = [];
				$scope.loadingSchedules = false;
				$scope.saveing = false;
				$scope.status = undefined;
				$scope.filter = 0;
				$scope.filterOptions = [
					{ label: $scope.LANG.t('All Schedules'), value: 0 },
					{ label: $scope.LANG.t('Manually Schedules'), value: 8 },
					{ label: $scope.LANG.t('Hourly Schedules'), value: 1 },
					{ label: $scope.LANG.t('Daily Schedules'), value: 2 },
					{ label: $scope.LANG.t('Weekly Schedules'), value: 3 },
					{ label: $scope.LANG.t('Monthly Schedules'), value: 4 },
					{ label: $scope.LANG.t('After Backup Job Schedules'), value: 5 },
					{ label: $scope.LANG.t('After Clone Job Schedules'), value: 6 },
					{ label: $scope.LANG.t('After cPanel Backup Done Schedules'), value: 7 }
				];

				$scope.loaders = {
					delete: false
				};

				meta = meta.new("schedules");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("desc");
				meta.setSortFields(["name","type","owner","jobs_count"]);
				meta.setTotalItems($scope.schedules.length);

				$scope.onClickDelete = function(schedule) {

					if($scope.saveing) return;

					if(schedule.jobs_count > 0)
					{
						SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("This schedule has an assigned jobs, Please remove those jobs before deleting this schedule."));
						return false;
					}

					$scope.saveing = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This schedule will be permanently deleted!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteSchedule({ _id: schedule._id }, function(response) {

								$scope.saveing = false;
								$scope.loaders.delete = false;

								if(response.success)
								{
									fetch();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Schedule delete cancelled"));
						}

					});
				};

				$scope.$on('fetch', fetch);

				function fetch() {

					$scope.loadingSchedules = true;
					$scope.status = undefined;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: {},
						filter: meta.getFilter()
					};

					if($scope.filter) apiParams.find.type = $scope.filter;
					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.schedules = [];

					jetapi.listSchedules(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.schedules);
						$scope.schedules = response.data.schedules;
						$scope.loadingSchedules = false;
					});
				}

				$timeout(fetch);
			}
		]
	);
});