'use strict';

define([
	'app',
	'libraries/permissions'
	], function(app, PERMISSIONS) {
	app.controller("security",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "jetapi", "meta", "SweetAlert", "utils", "lang", "cfpLoadingBar",
			function ($rootScope, $scope, $routeParams, $location, $timeout, jetapi, meta, SweetAlert, utils, lang, cfpLoadingBar) {
				$rootScope.$emit('menuItem', 'Security');

				if(!PERMISSIONS.isRoot) {
					$location.path("/");
					return;
				}

				$scope.saveing = false;
				$scope.status = undefined;
				$scope.loading = false;
				$scope.pluginTemplate = '';
				$scope.pluginsList = {};
				$scope.options = {};
				$scope.saveData = {
					plugin: '',
					options: {}
				};

				$scope.plugins = [
					{_id: '', name: $scope.LANG.t('Disabled')}
				];

				$scope.changePlugin = function() {
					if(!$scope.saveData.plugin) {
						$scope.pluginTemplate = '';
						return;
					}

					lang.loadNS('plugins-security-' + $scope.saveData.plugin);
					$scope.saveData.options = utils.duplicateObject($scope.pluginsList[$scope.saveData.plugin].options);
					$scope.pluginTemplate = 'templates/jetbackup/app/plugins/security/' + $scope.saveData.plugin + '/view.ptt';
				};

				$scope.$on('$destroy', function() {
					if($scope.changed)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.saveChanges = function(callback) {
					cfpLoadingBar.start();

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.duplicateObject($scope.saveData);
					apiParams.type = 'security';

					jetapi.managePlugin(apiParams, function(response) {

						$scope.saveing = false;

						if(response.success) {
							if(callback !== undefined) callback();
							SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
						}
						else {
							$scope.status = {
								message: response.message,
								type: "danger",
								closeable: false,
								ttl: 10000
							};
						}
						cfpLoadingBar.complete();
					});
				};



				$scope.fetch = function() {
					$scope.loading = true;
					jetapi.listPlugins({ type: 'security' }, function (response) {
						if(response.success) {
							for(var i = 0; i < response.data.plugins.length; i++) {
								$scope.plugins.push({_id:response.data.plugins[i]._id,name:response.data.plugins[i].name});
								$scope.pluginsList[response.data.plugins[i]._id] = response.data.plugins[i];
							}
						}

						jetapi.getSettings({ section: 'security' }, function(response) {
							$scope.loading = false;
							$scope.saveData.plugin = response.data.plugin;
							$scope.changePlugin();
						});
					});
				};

				$timeout($scope.fetch());

			}]);
});