'use strict';

define([
		'app',
		'libraries/consts',
],	function (app, CONSTS) {
		app.controller("wizardFileManager",
			["$rootScope", "$scope", "$interval", "$timeout", "jetapi", "meta", "SweetAlert", "$window", "$routeParams", "$location",
				function ($rootScope, $scope, $interval, $timeout, jetapi, meta, SweetAlert, $window, $routeParams, $location) {
					$rootScope.$emit('menuItem', 'BackupManager');

					if ($scope.service) $scope.service.showClose = true;

					$scope.type = "backup";
					$scope.typeId = $routeParams.id;
					$scope.typeDetails = {name: ''};
					$scope.files = [];
					$scope.loadingFiles = false;
					$scope.currentPath = '';
					$scope.breadcrumbs = [];
					$scope.filesIndex = {};
					$scope.actionModule = undefined;
					$scope.config = {};
					$scope.backup_type = 2;
					$scope.restoringBackup = false;
					$scope.isDirectories = false;
					$scope.showhidden = false;


					$scope.cancelAction = function () {
						$scope.actionStatus = undefined;
						$scope.actionModule = undefined;
					};

					$scope.changeView = function (view) {
						$location.path(view); // path not hash
					};

					$scope.onClickRestore = function () {
						var selected = getSelected();

						if (!selected.length) {
							$scope.actionStatus = {
								message: "Please select files to restore",
								type: "danger",
								closeable: true,
								ttl: 10000
							};
							return;
						}
						$scope.actionModule = 'restore';
					};

					$scope.$on('buildRestoreQuery', function () {
						$scope.restoringBackup = true;
						$scope.actionStatus = undefined;

						var apiParams = {};
						apiParams['_id'] = $scope.data.selectedSnap;
						apiParams['files'] = getSelected();

						$scope.$emit('onClickRestoreConfirm', apiParams);
						$scope.restoringBackup = false;
						$scope.actionModule = undefined;
						$scope.uncheckAll();
					});

					// $scope.onClickRestoreConfirm = function () {
					//
					// 	$scope.restoringBackup = true;
					// 	$scope.actionStatus = undefined;
					//
					// 	var apiParams = {};
					// 	apiParams['_id'] = $scope.data.selectedSnap;
					// 	apiParams['files'] = getSelected();
					//
					// 	$scope.$emit('onClickRestoreConfirm', apiParams);
					// 	$scope.restoringBackup = false;
					// 	$scope.actionModule = undefined;
					// 	$scope.uncheckAll();
					// };

					// $scope.isConditionsAgreed = function () {
					//
					// 	for (var i in $scope.config.restore_conditions) {
					// 		if ($scope.config.restore_conditions[i].type !== 0 && $scope.config.restore_conditions[i].type !== $scope.backup_type) continue;
					// 		if (!$scope.config.restore_conditions[i].checked) return true
					// 	}
					//
					// 	return false;
					// };

					var getSelected = function () {

						var selected = [];

						for (var i in $scope.filesIndex) {
							for (var j in $scope.filesIndex[i].data.files) {
								if ($scope.filesIndex[i].data.files[j] != null && $scope.filesIndex[i].data.files[j].checked !== undefined && $scope.filesIndex[i].data.files[j].checked) {
									selected.push($scope.filesIndex[i].data.files[j].path);
								}
							}
						}

						return selected;
					};

					$scope.canPerformAction = function () {

						for (var i in $scope.filesIndex) {
							for (var j in $scope.filesIndex[i].data.files) {
								if ($scope.filesIndex[i].data.files[j] != null && $scope.filesIndex[i].data.files[j].checked !== undefined && $scope.filesIndex[i].data.files[j].checked) {
									return false;
								}
							}
						}

						return true;
					};

					$scope.isDisabled = function () {
						var file = this.file.parent;
						while (!file.checked && file.parent) file = file.parent;
						return file.checked;
					};

					$scope.isChecked = function () {
						var file = this.file;
						while (!file.checked && file.parent) file = file.parent;
						return file.checked;
					};

					$scope.calculateInput = function (value) {

						if (value === undefined) {
							var file = this.file;
							while (!file.checked && file.parent) file = file.parent;
							return file.checked;
						}

						this.file.checked = value;
					};

					var manageBreadcrumbs = function (file) {
						if (file.path === '/') file.name = '/' + $scope.type + '-root';
						$scope.breadcrumbs.unshift(file);
						if (file.parent !== undefined) manageBreadcrumbs(file.parent);
					};

					var handleResponse = function (response, file) {
						$scope.filesIndex[file.path] = response;
						$scope.files = response.data.files;
						$scope.loadingFiles = false;
					};

					$scope.toggleHidden = function() {
						$scope.showhidden = !$scope.showhidden;
					};

					$scope.fetch = function (file) {

						jetapi.getConfigurations({}, function (response) {
							if (!response.success) return;
							$scope.config = response.data;
						});

						if (file.path === undefined) file.path = '/';

						$scope.loadingFiles = true;
						$scope.files = [];

						$scope.breadcrumbs = [];
						manageBreadcrumbs(file);

						if ($scope.filesIndex[file.path] !== undefined) {
							handleResponse($scope.filesIndex[file.path], file);
							return;
						}

						var apiParams = {};
						apiParams['_id'] = $scope.data.selectedSnap;
						apiParams['type'] = 'backup';
						apiParams['path'] = file.path;

						jetapi.fileManager(apiParams, function (response) {

							if (!response.success) {
								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								return false;
							}

							for (var i = 0; i < response.data.files.length; i++) {
								response.data.files[i].path = file.path + (file.path === '/' ? '' : '/') + response.data.files[i].name;
								response.data.files[i].parent = file;
								response.data.files[i].hidden = response.data.files[i].name.substr(0, 1) === '.';
							}

							handleResponse(response, file);
							return true;
						});

					};

					$scope.uncheckAll = function () {

						for (var i in $scope.filesIndex) {
							for (var j in $scope.filesIndex[i].data.files) {
								if ($scope.filesIndex[i].data.files[j] != null && $scope.filesIndex[i].data.files[j].checked !== undefined && $scope.filesIndex[i].data.files[j].checked) {
									$scope.filesIndex[i].data.files[j].checked = false;
								}
							}
						}

					};

					$scope.getDestination = function () {
						jetapi.getDestination({_id: $scope.typeId}, function (response) {

							if (!response.success) {
								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								return false;
							}

							$scope.typeDetails = response.data;
							return true;
						});
					};

					$scope.getBackup = function () {

						jetapi.getBackup({_id: $scope.data.selectedSnap}, function (response) {

							if (!response.success) {
								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								return false;
							}

							$scope.typeDetails = response.data;
							$scope.isDirectories = false;
							return true;
						});
					};

					$scope.init = function () {
						$scope.getBackup();
						$scope.fetch({path: '/'});
					};

					$timeout($scope.init());
				}]);
	}
);