'use strict';

define([
	'app',
	'libraries/consts'
], function (app, CONSTS) {
	app.controller("wizardFilters",
		["$rootScope", "$scope", "$location", "$timeout", "$uibModal", "jetapi","filterManager", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, $uibModal, jetapi, filterManager, SweetAlert) {

				if(!$scope.wizardData.jobId){
					SweetAlert.error($scope.LANG.t("Error!"), $scope.LANG.t("Can't fined backup job"));
					return;
				}

				$scope.data = { filters: [], selectedFilters: [] };
				$scope.data.filters.push({_id:'',name:'-- ' + $scope.LANG.t('SELECT FILTER') + ' -- '});


				$scope.initFilters = function (){
					$scope.newFilterFlag = false;

					jetapi.listAccountFilterGroups({},function(response) {
						if(!response.success) return; // TODO Handle Error
						for(var i = 0; i < response.data.groups.length; i++) {
							$scope.data.filters.push(response.data.groups[i]);
						}
					});
					$scope.preview.selectedFilter = $scope.data.filters.length ? $scope.data.filters[0]._id : '';

					if($scope.jobWizard && $scope.wizardData.filters) $scope.filtersStructure = filterManager.reBuildStructure($scope.wizardData.filters);
					else $scope.filtersStructure = [];
				}();


				$scope.$on($scope.wizardName, function () {
					jetapi.manageBackupJob({_id: $scope.wizardData.jobId, action: 'modify',	filters: $scope.data.selectedFilters}, function (response) {
						$scope.process(response);
					});
				});


				$scope.ShowFilters = function () {
					$scope.newFilterFlag = !$scope.newFilterFlag;
					if($scope.status) $scope.status = null;
				};

				$scope.$on('filterResponse', function (event, response) {

					if(!response.success) {
						$scope.status = {
							message: response.message,
							type: response.success ? "success" : "danger",
						};
						return;
					}
					jetapi.getAccountFilterGroup({ _id: response.data.group_id }, function(details) {
						if(details.success) {

							$scope.data.filters.push(details.data);
							$scope.preview.selectedFilter = details.data._id;
							$scope.add();
							$scope.status = null;
							$scope.newFilterFlag = false;

						}
						else {
							$scope.status = {
								message: details.message,
								type: "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});

				});

				$scope.newFilter = function () {
					$rootScope.$broadcast('createFilter');
				};

				$scope.add = function () {

					if (!$scope.preview.selectedFilter) return;

					var data = $scope.getFilterById($scope.preview.selectedFilter);
					if($scope.filtersStructure.length > 0) data.cond = 1;
					$scope.filtersStructure.push(data);
					$scope.preview.selectedFilter = $scope.data.filters[0]._id;
				};

				$scope.getFilterById = function(fId) {
					for(var i=0; i<$scope.data.filters.length; i++) {
						if ($scope.data.filters[i]._id === fId) return $scope.data.filters[i];
					}
					return false;
				};

				$scope.filtersConditions = [
					{_id:1,name:$scope.LANG.t("OR")},
					{_id:2,name:$scope.LANG.t("AND")}
				];


				$scope.$watch("filtersStructure", function(){
					$scope.buildFilters();
				}, true);

				$scope.removeFilter = function(index) {
					$scope.filtersStructure = filterManager.removeFilter($scope.filtersStructure, index);
				};

				$scope.upFilter = function(index) {
					if(index === 0) return;

					var clickedValue = $scope.filtersStructure[index];
					var upValue = $scope.filtersStructure[index-1];

					if(index === 1) {
						upValue.cond = clickedValue.cond;
						delete clickedValue.cond;
					}

					$scope.filtersStructure[index] = upValue;
					$scope.filtersStructure[index-1] = clickedValue;
				};

				$scope.downFilter = function(index) {
					if(index === ($scope.filtersStructure.length-1)) return;

					var clickedValue = $scope.filtersStructure[index];
					var downValue = $scope.filtersStructure[index+1];

					if(index === 0) {
						clickedValue.cond = downValue.cond;
						delete downValue.cond;
					}

					$scope.filtersStructure[index] = downValue;
					$scope.filtersStructure[index+1] = clickedValue;
				};

				$scope.buildFilters = function() {
					$scope.data.selectedFilters  = filterManager.buildFilters($scope.filtersStructure);
				};
			}
		]
	);
});