'use strict';

define([
	'app',
], function (app) {
	app.controller("wizardWelcomePage",
		["$rootScope", "$scope", "$timeout", "SweetAlert", "jetapi", "$uibModal",
			function ($rootScope, $scope, $timeout, SweetAlert, jetapi, $uibModal) {


				$scope.nameInterval = parseInt(new Date().getTime() % 10000);
				$scope.destinationParams = {
					action: "create",
					disasterrecovery: true,
					function: "manageDestination",
					destinationname: "Destination_" + $scope.nameInterval,
					jobname: "JetBackup_" + $scope.nameInterval,
					path: "/jetbackup",
					rsyncbwlimit: 0
				};
				$scope.responseData = {};

				var type = {local: 'localFields', ssh: 'sshFields'};

				$scope.createLocalJob = function () {
					resolveTemplateForm(type.local);
				};

				$scope.createSSHJob = function () {
					resolveTemplateForm(type.ssh);
				};

				var resolveTemplateForm = function (type) {
					$scope.model = $uibModal.open({
						animation: true,
						size: "lg",
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/wizard/' + type + '.ptt',
						controller: type,
						controllerAs: '$scope',
						resolve: {
							destinationParams: function () {
								return $scope.destinationParams;
							}
						}
					});

					$scope.model.result.then(function (data) {
						if (!data) return;

						$scope.responseData['destName'] = data.destinationname;

						createDailySchedule(function (schedules) {
							createAllAccountsBackupJob(data, schedules, function (response) {

								$scope.responseData['JobName'] = response.data.jobname;
								$scope.process({success: true, data: $scope.responseData, jobId: response.data._id});
							});
						});
					});
				};


				var createDailySchedule = function (callback) {

					var schedules = [];
					var scheduleParams = {
						action: "create",
						name: "daily_auto_" + parseInt(new Date().getTime() % 10000),
						type: 2,
						typedata: [1, 2, 3, 4, 5, 6, 7]
					};
					jetapi.manageSchedule(scheduleParams, function (response) {
						schedules.push({_id: response.data._id, name: response.data.name, retain: 14});
						callback(schedules);
					});
				};

				var createAllAccountsBackupJob = function (destinationParams, schedules, callback) {
					var jobParams = {
						action: "create",
						backuptypes: 127,
						destination: destinationParams._id,
						disabled: 0,
						flag: 1,//means incremental backup
						name: $scope.destinationParams.jobname,
						time: 100,
						schedules: schedules
					};
					jetapi.manageBackupJob(jobParams, function (response) {
						callback(response);
					});
				};


				$scope.goToWizard = function () {
					if (!$scope.jobWizard) $scope.process({success: false});
					else $scope.process({success: true, next:true});
				};


				$scope.$on($scope.wizardName, function (env, data) {
					$scope.process({success: true});
				});

			}
		]);
});