'use strict';

define([
	'app',
	'libraries/permissions'
], function (app, PERMISSIONS) {
	app.factory('jetapi', ["$rootScope", "$http", "$location", "cfpLoadingBar", "PermPermissionStore", "lang", function ($rootScope, $http, $location, cfpLoadingBar, PermPermissionStore, lang) {

		function exec(cmd, apiParams, callback, params) {

			if (!apiParams) apiParams = {};
			apiParams.function = cmd;

			if(apiParams.withLoader !== undefined && apiParams.withLoader) cfpLoadingBar.start();

			var request = $http.post('cgi/addons/jetbackup/api.cgi', apiParams);

			request.then(function(response) {
			
				if (response.data.message ) {
					var _data = [ 'exceptions|:|' + response.data.message ];
					if(!response.data.success && response.data.data.length) {
						for(var j=0; j<response.data.data.length; j++) {
							_data.push(response.data.data[j]); 
						}
					}
					response.data.message = lang.t.apply(lang, _data);
				}

				if(response.data.system.licenseIssue || response.data.system.agreement || response.data.system.drMode) {

					var location = '';
					window.PAGE.system = response.data.system;

					if(response.data.system.drMode) location = '/disasterRecovery';
					if(response.data.system.agreement) location = '/agreement';
					if(response.data.system.licenseIssue) location = '/license';

					PERMISSIONS._init();

					for(var i in PERMISSIONS) {
						if(!/^can/.test(i) && !/^is/.test(i) && !/^has/.test(i)) continue;
						PermPermissionStore.definePermission(i, function(name) { return PERMISSIONS[name]; });
					}

					$location.path(location);
				}

				if(apiParams.withLoader !== undefined && apiParams.withLoader) cfpLoadingBar.complete();
				if(callback !== undefined && typeof callback === 'function') callback(response.data, params);
			});

			return request;
		}

		return {
			// settings functions
			getSettings: function (apiParams, callback, params) {
				var cmd = 'getSettings' + apiParams.section[0].toUpperCase()+apiParams.section.substr(1);
				delete apiParams.section;
				return exec(cmd, apiParams, callback, params);
			},
			manageSettings: function (apiParams, callback, params) {
				var cmd = 'manageSettings' + apiParams.section[0].toUpperCase()+apiParams.section.substr(1);
				delete apiParams.section;
				return exec(cmd, apiParams, callback, params);
			},

			// log functions
			getLog: function (apiParams, callback, params) { return exec('getLog', apiParams, callback, params); },
			deleteLogs: function (apiParams, callback, params) { return exec('deleteLogs', apiParams, callback, params); },
			listLogs: function (apiParams, callback, params) { return exec('listLogs', apiParams, callback, params); },

			// account functions
			getAccountDetails: function (apiParams, callback, params) { return exec('getAccountDetails', apiParams, callback, params); },
			getAccount: function (apiParams, callback, params) { return exec('getAccount', apiParams, callback, params); },
			listAccounts: function (apiParams, callback, params) { return exec('listAccounts' + (apiParams.orphan ? 'Orphans' : '') + (apiParams.dr ? 'DisasterRecovery' : ''), apiParams, callback, params); },
			listAccountsBackups: function (apiParams, callback, params) { return exec('listAccountsBackups', apiParams, callback, params); },
			manageAccount: function (apiParams, callback, params) { return exec('manageAccount', apiParams, callback, params); },
			listAccountsByBackupFilters: function (apiParams, callback, params) { return exec('listAccountsByBackupFilters', apiParams, callback, params); },
			//listAccountsWithoutBackupJob: function (apiParams, callback, params) { return exec('listAccountsWithoutBackupJob', apiParams, callback, params); },

			// backup functions
			deleteBackupJob: function (apiParams, callback, params) { return exec('deleteBackupJob', apiParams, callback, params); },
			getBackupJob: function (apiParams, callback, params) { return exec('getBackupJob', apiParams, callback, params); },
			duplicateBackupJob: function (apiParams, callback, params) { return exec('duplicateBackupJob', apiParams, callback, params); },
			listBackupJobs: function (apiParams, callback, params) { return exec('listBackupJobs', apiParams, callback, params); },
			getBackup: function (apiParams, callback, params) { return exec('getBackup', apiParams, callback, params); },
			listBackups: function (apiParams, callback, params) { return exec('listBackups', apiParams, callback, params); },
			manageBackupJob: function (apiParams, callback, params) { return exec('manageBackupJob', apiParams, callback, params); },
			addQueueRestore: function (apiParams, callback, params) { return exec('addQueueRestore', apiParams, callback, params); },
			addQueueSnapshot: function (apiParams, callback, params) { return exec('addQueueSnapshot', apiParams, callback, params); },
			calculateBackupJobRating: function (apiParams, callback, params) { return exec('calculateBackupJobRating', apiParams, callback, params); },
			calculateBackupSize: function (apiParams, callback, params) { return exec('calculateBackupSize', apiParams, callback, params); },
			setBackupJobNextRun: function (apiParams, callback, params) { return exec('setBackupJobNextRun', apiParams, callback, params); },
			manageBackup: function (apiParams, callback, params) { return exec('manageBackup', apiParams, callback, params); },
			stopBackupJob: function (apiParams, callback, params) { return exec('stopBackupJob', apiParams, callback, params); },
			deleteDownload: function (apiParams, callback, params) { return exec('deleteDownload', apiParams, callback, params); },
			getBackupDownloads: function (apiParams, callback, params) { return exec('getBackupDownloads', apiParams, callback, params); },
			addQueueDownload: function (apiParams, callback, params) { return exec('addQueueDownload', apiParams, callback, params); },
			queueRetry: function(apiParams, callback, params) {return exec('queueRetry', apiParams, callback, params); },
			listPackages: function (apiParams, callback, params) { return exec('listPackages', apiParams, callback, params); },
			getQueueItem: function (apiParams, callback, params) { return exec('getQueueItem', apiParams, callback, params); },
			listQueueItems: function (apiParams, callback, params) { return exec('listQueueItems', apiParams, callback, params); },
			cancelQueueItem: function (apiParams, callback, params) { return exec('cancelQueueItem', apiParams, callback, params); },
			listDirectories: function (apiParams, callback, params) { return exec('listDirectories', apiParams, callback, params); },

			// Security Addons
			listPlugins: function (apiParams, callback, params) { return exec('listPlugins', apiParams, callback, params); },
			getPlugin: function (apiParams, callback, params) { return exec('getPlugin', apiParams, callback, params); },
			managePlugin: function (apiParams, callback, params) { return exec('managePlugin', apiParams, callback, params); },

			// clone functions
			deleteCloneJob: function (apiParams, callback, params) { return exec('deleteCloneJob', apiParams, callback, params); },
			addQueueClone: function (apiParams, callback, params) { return exec('addQueueClone', apiParams, callback, params); },
			getCloneJob: function (apiParams, callback, params) { return exec('getCloneJob', apiParams, callback, params); },
			listCloneJobs: function (apiParams, callback, params) { return exec('listCloneJobs', apiParams, callback, params); },
			manageCloneJob: function (apiParams, callback, params) { return exec('manageCloneJob', apiParams, callback, params); },
			duplicateCloneJob: function (apiParams, callback, params) { return exec('duplicateCloneJob', apiParams, callback, params); },
			setCloneJobNextRun: function (apiParams, callback, params) { return exec('setCloneJobNextRun', apiParams, callback, params); },
			stopCloneJob: function (apiParams, callback, params) { return exec('stopCloneJob', apiParams, callback, params); },

			// restore conditions functions
			deleteRestoreCondition: function (apiParams, callback, params) { return exec('deleteRestoreCondition', apiParams, callback, params); },
			getRestoreCondition: function (apiParams, callback, params) { return exec('getRestoreCondition', apiParams, callback, params); },
			listRestoreConditions: function (apiParams, callback, params) { return exec('listRestoreConditions', apiParams, callback, params); },
			manageRestoreCondition: function (apiParams, callback, params) { return exec('manageRestoreCondition', apiParams, callback, params); },

			// queue priorities functions
			deleteQueuePriority: 	 function (apiParams, callback, params) { return exec('deleteQueuePriority', apiParams, callback, params); },
			getQueuePriority: 	 	 function (apiParams, callback, params) { return exec('getQueuePriority', apiParams, callback, params); },
			listQueuePriorities: 	 function (apiParams, callback, params) { return exec('listQueuePriorities', apiParams, callback, params); },
			manageQueuePriority: 	 function (apiParams, callback, params) { return exec('manageQueuePriority', apiParams, callback, params); },
			setDefaultQueuePriority: function (apiParams, callback, params) { return exec('setDefaultQueuePriority', apiParams, callback, params); },
			changeQueueItemPriority: function (apiParams, callback, params) { return exec('changeQueueItemPriority', apiParams, callback, params); },

			// file permissions functions
			deleteFilePermission: function (apiParams, callback, params) { return exec('deleteFilePermission', apiParams, callback, params); },
			getFilePermission: function (apiParams, callback, params) { return exec('getFilePermission', apiParams, callback, params); },
			listFilePermissions: function (apiParams, callback, params) { return exec('listFilePermissions', apiParams, callback, params); },
			manageFilePermission: function (apiParams, callback, params) { return exec('manageFilePermission', apiParams, callback, params); },

			// schedule functions
			deleteSchedule: function (apiParams, callback, params) { return exec('deleteSchedule', apiParams, callback, params); },
			getSchedule: function (apiParams, callback, params) { return exec('getSchedule', apiParams, callback, params); },
			listSchedules: function (apiParams, callback, params) { return exec('listSchedules', apiParams, callback, params); },
			manageSchedule: function (apiParams, callback, params) { return exec('manageSchedule', apiParams, callback, params); },

			// hook functions
			deleteHook: function (apiParams, callback, params) { return exec('deleteHook', apiParams, callback, params); },
			getHook: function (apiParams, callback, params) { return exec('getHook', apiParams, callback, params); },
			listHooks: function (apiParams, callback, params) { return exec('listHooks', apiParams, callback, params); },
			manageHook: function (apiParams, callback, params) { return exec('manageHook', apiParams, callback, params); },

			// account filters functions
			deleteAccountFilter: function (apiParams, callback, params) { return exec('deleteAccountFilter', apiParams, callback, params); },
			getAccountFilter: function (apiParams, callback, params) { return exec('getAccountFilter', apiParams, callback, params); },
			listAccountFilters: function (apiParams, callback, params) { return exec('listAccountFilters', apiParams, callback, params); },
			manageAccountFilter: function (apiParams, callback, params) { return exec('manageAccountFilter', apiParams, callback, params); },
			deleteAccountFilterGroup: function (apiParams, callback, params) { return exec('deleteAccountFilterGroup', apiParams, callback, params); },
			getAccountFilterGroup: function (apiParams, callback, params) { return exec('getAccountFilterGroup', apiParams, callback, params); },
			listAccountFilterGroups: function (apiParams, callback, params) { return exec('listAccountFilterGroups', apiParams, callback, params); },
			manageAccountFilterGroup: function (apiParams, callback, params) { return exec('manageAccountFilterGroup', apiParams, callback, params); },
			updateAccountsBackupStats: function (apiParams, callback, params) { return exec('updateAccountsBackupStats', apiParams, callback, params); },

			// destination functions
			deleteDestination: function (apiParams, callback, params) { return exec('deleteDestination', apiParams, callback, params); },
			getDestination: function (apiParams, callback, params) { return exec('getDestination', apiParams, callback, params); },
			listDestinations: function (apiParams, callback, params) { return exec('listDestinations', apiParams, callback, params); },
			manageDestination: function (apiParams, callback, params) { return exec('manageDestination', apiParams, callback, params); },
			validateDestination: function (apiParams, callback, params) { return exec('validateDestination', apiParams, callback, params); },

			// alert functions
			listAlerts: function (apiParams, callback, params) { return exec('listAlerts', apiParams, callback, params); },
			acknowledgeAlerts: function (apiParams, callback, params) { return exec('acknowledgeAlerts', apiParams, callback, params); },

			// permissions functions
			getPermissions: function (apiParams, callback, params) { return exec('getPermissions', apiParams, callback, params); },
			managePermissions: function (apiParams, callback, params) { return exec('managePermissions', apiParams, callback, params); },
			resetPermissions: function (apiParams, callback, params) { return exec('resetPermissions', apiParams, callback, params); },
			listPermissions: function (apiParams, callback, params) { return exec('listPermissions', apiParams, callback, params); },

			// dr
			restoreDisasterRecovery: function (apiParams, callback, params) { return exec('restoreDisasterRecovery', apiParams, callback, params); },
			exitDisasterRecovery: function (apiParams, callback, params) { return exec('exitDisasterRecovery', apiParams, callback, params); },
			factoryReset: function (apiParams, callback, params) { return exec('factoryReset', apiParams, callback, params); },
			exitDisasterRecoveryKeepData: function (apiParams, callback, params) { return exec('exitDisasterRecoveryKeepData', apiParams, callback, params); },


			// util functions
			getInfo: function (apiParams, callback, params) { return exec('getInfo', apiParams, callback, params); },
			reindex: function (apiParams, callback, params) { return exec('reindex', apiParams, callback, params); },
			approveShowcase: function (apiParams, callback, params) { return exec('approveShowcase', apiParams, callback, params); },
			approveAgreement: function (apiParams, callback, params) { return exec('approveAgreement', apiParams, callback, params); },
			generateRootEncryptionKey: function (apiParams, callback, params) { return exec('generateRootEncryptionKey', apiParams, callback, params); },
			disconnectManagementConsole: function (apiParams, callback, params) { return exec('disconnectManagementConsole', apiParams, callback, params); },
			getConfigurations: function (apiParams, callback, params) { return exec('getConfigurations', apiParams, callback, params); },
			getDashboardDetails: function (apiParams, callback, params) { return exec('getDashboardDetails', apiParams, callback, params); },
			getStatistics: function (apiParams, callback, params) { return exec('getStatistics', apiParams, callback, params); },
			resetEncryptionKey: function (apiParams, callback, params) { return exec('resetEncryptionKey', apiParams, callback, params); },
			fileManager: function (apiParams, callback, params) { return exec('fileManager', apiParams, callback, params); },
			stopProcess: function (apiParams, callback, params) { return exec('stopProcess', apiParams, callback, params); }
		};
	}]);
});